//
// This is the GameState class.  It contains all the states of this game, and is
// essentially our 'global variables' for the game, so to speak.
//
#include "GameState.h"

GameState::GameState()
{
	human_board = NULL;
	ai_board = NULL;
	ai_shots = 0;
	human_shots = 0;
	units_placed = 0;
	ai_units_placed = 0;
	ai_targets = NULL;
	human_targets = NULL;
	cout << "Initialized GameState!" << endl;
}

// coord2Index(x,y)
//
// We'll want to use x,y coordinates...but our board is
// stored as a 1-dimensional array.
//
// Coordinates are from (0,0) to (BOARD_WIDTH-1,BOARD_HEIGHT-1)
//
short coord2Index(int x, int y)
{
	return (y * BOARD_WIDTH + x);
}

// get()
//
// Simply returns the current game state.
//
short GameState::get()
{
	return current;
}

// getCoord(x,y)
//
// Simply returns the coordinate values
//
b_coord *GameState::getHumanCoord(int x, int y)
{
	int idx = coord2Index(x, y);

	return &human_board[idx];
}

// getCoord(x,y)
//
// Simply returns the coordinate values
//
b_coord *GameState::getAiCoord(int x, int y)
{
	int idx = coord2Index(x, y);

	return &ai_board[idx];
}

// set()
//
// Simply returns the current game state.
//
void GameState::set(short gs)
{
	current = gs;
}

// fireAtAi(x,y)
//
// This will return a 'true' if the AI has a ship at the
// coordinate (x,y).  Also, it will update the board to
// reflect whether that coordinate.dead is now 'true'.
//
bool GameState::fireAtAi(int x, int y)
{
	int idx = coord2Index(x, y);
	bool ret_val = false;

	//cout << "Checking type of (" << x << ", " << y << ")" << endl;
	switch(ai_board[idx].type)
	{
	case GRID_BOW:
		ai_board[idx].dead = true;
		ai_targets[idx] = TARGET_HIT;
		ret_val = true;
		//cout << "Setting BOW (" << x << ", " << y << ") to TARGET_HIT" << endl;
		break;
	case GRID_1GUN:
		ai_board[idx].dead = true;
		ai_targets[idx] = TARGET_HIT;
		ret_val = true;
		//cout << "Setting 1GUN (" << x << ", " << y << ") to TARGET_HIT" << endl;
		break;
	case GRID_STERN:	
		ai_board[idx].dead = true;
		ai_targets[idx] = TARGET_HIT;
		ret_val = true;
		//cout << "Setting STERN(" << x << ", " << y << ") to TARGET_HIT" << endl;
		break;
	default:
		ai_targets[idx] = TARGET_TRIED;
		//cout << "Setting misc (" << x << ", " << y << ") to TARGET_TRIED" << endl;
	}
	return ret_val;
}

// fireAtHuman(x,y)
//
// This will return a 'true' if the Human has a ship at the
// coordinate (x,y).  Also, it will update the board to
// reflect whether that coordinate.dead is now 'true'.
//
bool GameState::fireAtHuman(int x, int y)
{
	int idx = coord2Index(x, y);

	if (human_board[idx].type == GRID_BOW ||
		human_board[idx].type == GRID_1GUN ||
		human_board[idx].type == GRID_2GUN ||
		human_board[idx].type == GRID_STERN)
	{
		human_board[idx].dead = true;
		human_targets[idx] = TARGET_HIT;
		return true;
	}
	else
	{
		human_targets[idx] = TARGET_TRIED;
		return false;
	}

}

// getAiTargetHistory(x,y)
//
// returns the status of AI's history (if any) with this
// particular Human grid coordinate.
//
short GameState::getAiTargetHistory(int x, int y)
{
	int idx = coord2Index(x, y);

	return human_targets[idx];
}

// getHumanTargetHistory(x,y)
//
// returns the status of Human's history (if any) with this
// particular Ai grid coordinate.
//
short GameState::getHumanTargetHistory(int x, int y)
{
	int idx = coord2Index(x, y);

	return ai_targets[idx];
}

// selectAiTarget(x,y)
//
// Human is picking targets to shoot at.  This will
// select a square for targeting.
//
bool GameState::selectAiTarget(int x, int y)
{
	int idx = coord2Index(x, y);

	if (human_shots > 0)
	{
		if (ai_targets[idx] == TARGET_AVAILABLE)
		{
			//cout << "Selecting (" << x << ", " << y << ")" << endl;
			// Select the target
			ai_targets[idx] = TARGET_SELECTED;
			// Decrease shots remaining
			human_shots--;
			return true;
		}
	}
	return false;
}

// deselectAiTarget(x,y)
//
// Human is picking targets to shoot at.  This will
// deselect a square for targeting.
//
bool GameState::deselectAiTarget(int x, int y)
{
	int idx = coord2Index(x, y);

	if (ai_targets[idx] == TARGET_SELECTED)
	{
		// Un-select the target
		ai_targets[idx] = TARGET_AVAILABLE;
		// Increase shots remaining
		human_shots++;
		return true;
	}
	return false;
}

// selectHumanTarget(x,y)
//
// AI is picking targets to shoot at.  This will set
// a square as selected for targeting.
//
bool GameState::selectHumanTarget(int x, int y)
{
	int idx = coord2Index(x, y);

	if (ai_shots > 0)
	{
		if (human_targets[idx] == TARGET_AVAILABLE)
		{
			// Select the target
			human_targets[idx] = TARGET_SELECTED;
			// Decrease shots remaining
			ai_shots--;
			return true;
		}
		return false;
	}
	return false;
}

// deselectHumanTarget(x,y)
//
// AI is picking targets to shoot at.  This will
// deselect a square for targeting.
//
bool GameState::deselectHumanTarget(int x, int y)
{
	int idx = coord2Index(x, y);

	if (human_targets[idx] == TARGET_SELECTED)
	{
		// Un-select the target
		human_targets[idx] = TARGET_AVAILABLE;
		// Increase shots remaining
		ai_shots++;
		return true;
	}
	return false;
}


// getHumanShotsRemaining()
//
// Simply returns the number of shots remaining for
// the human player
//
short GameState::getHumanShotsRemaining()
{
	return human_shots;
}

// getAiShotsRemaining()
//
// Simply returns the number of shots remaining for
// the AI player
//
short GameState::getAiShotsRemaining()
{
	return ai_shots;
}

// resetHumanShotsRemaining
//
// This goes through the board, and sees which guns are
// operational.  (They'll be GRID_DEAD, if damaged.)
//
void GameState::resetHumanShotsRemaining()
{
	int i;

	human_shots = 0;

	for(i=0; i < BOARD_WIDTH * BOARD_LENGTH; i++)
	{
		if (human_board[i].type == GRID_1GUN)
		{
			if (!human_board[i].dead)
				human_shots += 1;
		}
	}
}

// resetAiShotsRemaining
//
// This goes through the board, and sees which guns are
// operational.  (They'll be GRID_DEAD, if damaged.)
//
void GameState::resetAiShotsRemaining()
{
	int i;

	ai_shots = 0;

	for (i=0; i < BOARD_WIDTH * BOARD_LENGTH; i++)
	{
		if (ai_board[i].type == GRID_1GUN)
		{
			if (!ai_board[i].dead)
				ai_shots += 1;
		}
	}
}

void GameState::setHumanTurn()
{
	human_turn = true;
}

void GameState::setAiTurn()
{
	human_turn = false;
}

bool GameState::initAiBoardFromFile(char *filename)
{
	//FILE *filePtr = NULL;
	ifstream filePtr;

	int width = BOARD_WIDTH;
	int length = BOARD_LENGTH;
	int i = 0;
	int j = 0;
	int idx = 0;
	short f = 0;		// temporary for 'facing'
	short t = 0;		// temporary for 

	// If we have an old board...kill it.
	if (ai_board != NULL)
		free(ai_board);

	// open board setup file.
	filePtr.open(filename, ios::in);
	if (!filePtr.is_open())
	{
		//printf("Couldn't open %s\n", filename);
		cout << "Couldn't open " << filename << endl;
		return false;
	}
	else 
	{
		//printf("opened %s\n", filename);
		cout << "opened " << filename << endl;
	}

	//fscanf(filePtr, "%d %d", &width, &length);
	filePtr >> width >> length;

	// re-initialize the board
	ai_board = new b_coord[length*width];
	ai_targets = new short[length*width];


	if (width != BOARD_WIDTH || length != BOARD_LENGTH)
		return false;

	for (i=0; i < length; i++)
	{
		for (j=0; j < width; j++)
		{
			idx = coord2Index(j,i);
			filePtr >> ai_board[idx].facing >> ai_board[idx].type;
			ai_board[idx].dead = false;
			//cout << "AI loaded {" << j << "," << i << ") as (" << ai_board[idx].facing << "," << ai_board[idx].type << ")" << endl;
			if (ai_board[idx].type != GRID_LAND)
				ai_targets[idx] = TARGET_AVAILABLE;
			else
				ai_targets[idx] = TARGET_UNAVAILABLE;
		}
	}
	//fclose(filePtr);
	filePtr.close();
	return true;
}


bool GameState::initAiRandBoard()
{

	while (ai_units_placed <= MAX_OCCUPANCY - 3) {

		int rand_grid = (int)(0 + (400 * rand() / (1.0 + RAND_MAX)));
		int rand_direction = (int)(0 + (360 * rand() / (1.0 + RAND_MAX)));
		int rand_size = (int)(3 + (8 * rand() / (1.0 + RAND_MAX)));

		if (rand_direction < 90) {
			rand_direction = 0;
		}
		else if (rand_direction < 180)
		{
			rand_direction = 90;
		}
		else if (rand_direction < 270)
		{
			rand_direction = 180;
		}
		else
		{
			rand_direction = 270;
		}

		//cout << "Generated random ship: " << rand_grid << " " << rand_direction << " " << rand_size << endl;
		
		if (isValidPlacement(rand_grid, rand_direction, rand_size, AI))
		{
			cout << "AI units placed: " << ai_units_placed << endl;
			placeShipOnBoard(rand_grid, rand_direction, rand_size, AI);
		}

	}

	return true;
}


bool GameState::initHumanBoardFromFile(char *filename)
{
	//FILE *filePtr = NULL;
	ifstream filePtr;
	
	int width = BOARD_WIDTH;
	int length = BOARD_LENGTH;
	int i = 0;
	int j = 0;
	int idx = 0;
	short f = 0;		// temporary for 'facing'
	short t = 0;		// temporary for 

	// If we have an old board...kill it.
	if (human_board != NULL)
		free(human_board);

	// open board setup file.
	filePtr.open(filename, ios::in);
	if (!filePtr.is_open())
	{
		//printf("Couldn't open %s\n", filename);
		cout << "Couldn't open " << filename << endl;
		return false;
	}
	else 
	{
		//printf("opened %s\n", filename);
		cout << "opened " << filename << endl;
	}

	//fscanf(filePtr, "%d %d", &width, &length);
	filePtr >> width >> length;

	// re-initialize the board
	human_board = new b_coord[length*width];
	human_targets = new short[length*width];


	if (width != BOARD_WIDTH || length != BOARD_LENGTH)
		return false;

	for (i=0; i < length; i++)
	{
		for (j=0; j < width; j++)
		{
			idx = coord2Index(j,i);
			filePtr >> human_board[idx].facing >> human_board[idx].type;
			human_board[idx].dead = false;
			//cout << "HUMAN loaded {" << j << "," << i << ") as (" << human_board[idx].facing << "," << human_board[idx].type << ")" << endl;
			if (human_board[idx].type != GRID_LAND)
				human_targets[idx] = TARGET_AVAILABLE;
			else
				human_targets[idx] = TARGET_UNAVAILABLE;
		}
	}
	//fclose(filePtr);
	filePtr.close();
	return true;
}


bool GameState::initAiBoard()
{
	int i = 0;
	int j = 0;
	int idx = 0;

	// If we have an old board...kill it.
	if (ai_board != NULL)
		free(ai_board);

	// re-initialize the board
	ai_board = (b_coord *)malloc(sizeof(b_coord)*BOARD_LENGTH*BOARD_WIDTH);
	ai_targets = (short *)malloc(sizeof(short)*BOARD_LENGTH*BOARD_WIDTH);

	for (i=0; i < BOARD_LENGTH; i++)
	{
		for (j=0; j < BOARD_WIDTH; j++)
		{
			idx = coord2Index(j,i);
			ai_board[idx].facing = 0;
			ai_board[idx].type = 0;
			ai_board[idx].dead = false;
			ai_targets[idx] = TARGET_AVAILABLE;
		}
	}
	return true;
}

bool GameState::initHumanBoard()
{
	int i = 0;
	int j = 0;
	int idx = 0;

	// If we have an old board...kill it.
	if (human_board != NULL)
		free(human_board);

	// re-initialize the board
	human_board = (b_coord *)malloc(sizeof(b_coord)*BOARD_LENGTH*BOARD_WIDTH);
	human_targets = (short *)malloc(sizeof(short)*BOARD_LENGTH*BOARD_WIDTH);

	for (i=0; i < BOARD_LENGTH; i++)
	{
		for (j=0; j < BOARD_WIDTH; j++)
		{
			idx = coord2Index(j,i);
			human_board[idx].facing = 0;
			human_board[idx].type = 0;
			human_board[idx].dead = false;
			human_targets[idx] = TARGET_AVAILABLE;
		}
	}
	return true;
}


bool GameState::placeShipOnBoard(int grid, int direction, int size, int board_type)
{
	int next = 0;

	//cout << "Placing Ship" << endl;
	//cout << "Direction:" << direction << endl;
	//cout << "Grid:" << grid << endl;
	//cout << "Size:" << size << endl << endl;

	if (board_type == HUMAN)
	{
		// there is no board neo
		if (human_board == NULL)
			return false;

		for (int x = 0; x < size; x++){

			// place the piece
			if (x == 0)
			{
				human_board[grid + next].type = GRID_BOW;
			} 
			else if (x < (size - 1))
			{
				human_board[grid + next].type = GRID_1GUN;
			}
			else
			{
				human_board[grid + next].type = GRID_STERN;
			}

			// set the direction
			if (direction == 0) {
				human_board[grid + next].facing = NORTH;
				next += 20;
			}
			else if (direction == 90) {
				human_board[grid + next].facing = WEST;
				next += 1;
			}
			else if (direction == 180) {
				human_board[grid + next].facing = SOUTH;
				next -= 20;
			}
			else if (direction == 270) {
				human_board[grid + next].facing = EAST;
				next -= 1;
			}

			//cout << "Next piece at: " << grid + next << endl;
		}

		units_placed += size;

		cout << "units placed: " << units_placed << endl;

	}
	else if (board_type == AI)
	{
		// there is no board neo
		if (ai_board == NULL)
			return false;

		for (int x = 0; x < size; x++){

			// place the piece
			if (x == 0)
			{
				ai_board[grid + next].type = GRID_BOW;
			} 
			else if (x < (size - 1))
			{
				ai_board[grid + next].type = GRID_1GUN;
			}
			else
			{
				ai_board[grid + next].type = GRID_STERN;
			}

			// set the direction
			if (direction == 0) {
				ai_board[grid + next].facing = NORTH;
				next += 20;
			}
			else if (direction == 90) {
				ai_board[grid + next].facing = WEST;
				next += 1;
			}
			else if (direction == 180) {
				ai_board[grid + next].facing = SOUTH;
				next -= 20;
			}
			else if (direction == 270) {
				ai_board[grid + next].facing = EAST;
				next -= 1;
			}

			//cout << "Next piece at: " << grid + next << endl;
		}

		ai_units_placed += size;

		cout << "AI units placed: " << ai_units_placed << endl;

	}

	return true;
}


bool GameState::isValidPlacement(int grid, int direction, int size, int board_type)
{
	int next = 0;
	int row = grid / 20;
	int current = 0;

	if (board_type == HUMAN) {
		// there is no board neo
		if (human_board == NULL)
			return false;

		if (units_placed + size > MAX_OCCUPANCY)
			return false;
	}
	else
	{
		// there is no board neo
		if (ai_board == NULL)
			return false;

		if (ai_units_placed + size > MAX_OCCUPANCY)
			return false;
	}


	for (int x = 0; x < size; x++) {

		current = grid + next;

		//cout << "evaluating grid space: " << current << endl;

		if (board_type == HUMAN) {
			//cout << "HUMAN grid->type: " <<  human_board[current].type << endl;
			if (human_board[current].type != GRID_WATER)
				return false;
		}
		if (board_type == AI) {
			//cout << "AI grid->type: " <<  ai_board[current].type << endl;
			if (ai_board[current].type != GRID_WATER)
				return false;
		}


		if (current > 399 || current < 0)
			return false;
		
		// Next ship piece would be where?
		if (direction == 0) {
			next += 20;
		}
		else if (direction == 90) {
			next += 1;
		}
		else if (direction == 180) {
			next -= 20;
		}
		else if (direction == 270) {
			next -= 1;
		}
	}

	return true;
}

bool GameState::allUnitsPlaced(void)
{
	if (units_placed > MAX_OCCUPANCY - 3)
		return true;

	return false;
}


void GameState::damageHumanBoardPiece(int x, int y)
{
	int idx = 0;

	idx = coord2Index(x,y);

	human_board[idx].dead = true;
}

void GameState::damageAiBoardPiece(int x, int y)
{
	int idx = 0;

	idx = coord2Index(x,y);

	ai_board[idx].dead = true;
}
