#ifndef __GAMESTATE_H
#define __GAMESTATE_H

// Includes
//
#include <iostream>
#include <fstream>
using namespace std;
//#include <stdio.h>
//#include <stdlib.h>

#define MAX_OCCUPANCY 40	// Number of squares player can occupy in grid
#define BOARD_WIDTH   20	// Number of squares across the grid
#define BOARD_LENGTH  20	// Number of squares down the grid

#define STATE_MENU					1
#define STATE_SETUP					2
#define STATE_TARGETING				3
#define STATE_BATTLE				4
#define STATE_BINOCULARS			5
#define STATE_STATISTICS			6
#define STATE_CREDITS				7
#define STATE_FIRING				8
#define STATE_FIRING_AND_BINOCULARS 9

// b_coord facing
//
// Tells us the direction the bow of the ship faces.
#define NORTH 0
#define EAST  1
#define SOUTH 2
#define WEST  3

// b_coord types
//
#define GRID_WATER 0			// This coordinate contains water only
#define GRID_LAND  1			// This coordinate contains land only
#define GRID_BOW   2			// This coordinate contains the bow of a ship
#define GRID_1GUN  3			// This coordinate contains 1-gun ship component
#define GRID_2GUN  4			// This coordinate contains a 2-gun ship component
#define GRID_STERN 5			// This coordinate contains the stern of the ship

// Target states
//
#define TARGET_UNAVAILABLE -1	// Target is unavailable.
#define TARGET_AVAILABLE    0	// Target is available for targeting
#define TARGET_SELECTED     1	// Target is selected for firing
#define TARGET_TRIED        2	// Target was selected in previous rounds
#define TARGET_HIT          3	// Target was selected in previous rounds


enum board_type { HUMAN, AI };

// b_coord
//
// This is a coordinate on the board.

struct b_coord
{
	short facing;
	short type;
	bool dead;
};

// GameState
//
// This is the class that maintains all data in the game.
//
class GameState
{
private:
	short current;	// Current state
	b_coord	*human_board;
	b_coord *ai_board;
	short ai_shots;
	short human_shots;
	short *ai_targets;
	short *human_targets;
	bool human_turn;
	int units_placed;
	int ai_units_placed;

public:

	GameState();
	short get();	// Return the current game state
	b_coord *getHumanCoord(int, int);
	b_coord *getAiCoord(int, int);
	short getHumanTargetHistory(int, int);
	short getAiTargetHistory(int, int);
	void set(short);
	void setHumanTurn();
	void setAiTurn();
	bool fireAtAi(int, int);
	bool fireAtHuman(int, int);
	bool selectAiTarget(int, int);
	bool selectHumanTarget(int, int);
	bool deselectAiTarget(int, int);
	bool deselectHumanTarget(int, int);
	short getHumanShotsRemaining();
	short getAiShotsRemaining();
	void resetHumanShotsRemaining();
	void resetAiShotsRemaining();
	bool initAiBoardFromFile(char *);
	bool initAiRandBoard(void);
	bool initHumanBoardFromFile(char *);
	bool placeShipOnBoard(int, int, int, int);
	bool isValidPlacement(int, int, int, int);
	bool allUnitsPlaced();
	bool initAiBoard();		// Initialize land/water (no pieces yet)
	bool initHumanBoard();	// Initialize land/water (no pieces yet)
	void damageHumanBoardPiece(int, int);	// use b_coord to update board coordinate
	void damageAiBoardPiece(int, int);		// use b_coord to update board coordinate
};

#endif