////// Includes
#include <iostream>
#include <cmath>
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glaux.h>
#include <time.h>
#include "somepoint.h"
#include "somevector.h"
#include "somefont.h"
#include "texture.h"
#include "init.h"
#include "HiResTimer.h"
#include "3ds.h"
#include "GameState.h"
#include "Cheats.h"
#include "model.h"
#include "harborwars.h"
#include "missile.h"

////// Global Variables

// GameState object
GameState *g_state = NULL;		// gamestate class.

CHiResTimer		g_timer;		// High Resolution Timer

Model  ship_bow;
Model  ship_1gun_w;
Model  ship_1gun_s;
Model  ship_1gun_e;
Model  ship_1gun_n;
Model  ship_stern;
Model  busted_bow;
Model  busted_1gun;
Model  busted_stern;
Model  missile;
Model  missile2;

Texture * texture_sky;
Texture * texture_land;
Texture * texture_water;
Texture * b_target;
Texture * b_surrender;
Texture * b_binoculars;
Texture * b_shots;
Texture * b_hits;
Texture * b_fire;
Texture * binoculars;
Texture * s_done;

TGAFILE * img_hud;
tImageTGA * img_hud2;
BITMAPINFOHEADER hud_header;
unsigned char * img_hud3;

int tmpx, tmpy;				// mouse cords


// terrain data
GLuint  g_indexArray[MAP_X * MAP_Z * 6];    // vertex index array
float   g_terrain[MAP_X * MAP_Z][3];         // heightfield terrain data (0-255); 256x256
float   g_colorArray[MAP_X * MAP_Z][3];     // color array
float   g_texcoordArray[MAP_X * MAP_Z][2];  // tex coord array
float	g_normalFaceData[(MAP_X-1) * (MAP_Z-1)][3];
float	g_normalData[MAP_X * MAP_Z][3];
GLuint	landCallList;
GLuint	waterCallList;

float zoom = 0.0f;									// hold the zoom
float zoom_total = -35.0f;							// zoom total for window resizing
float battle_yaw = 0.0f;
float battle_pitch = 0.0f;
float battle_zoom = 0.0f;
float eye_height = 25.0f;

HDC	g_HDC;											// global device context
somepoint mouse_point1;
somefont	test_font;

int num_human_turrets = 0;
int num_ai_turrets = 0;
int num_human_targets = 0;
int num_ai_targets = 0;
int rand_human_turrets[MAX_TURRETS];
int num_ai_hits = 0;
int num_human_hits = 0;
t_list human_gun[MAX_TURRETS];
t_list ai_gun[MAX_TURRETS];
somepoint human_target_list[MAX_TURRETS];
somepoint ai_target_list[MAX_TURRETS];
Missile human_missiles[MAX_TURRETS];
Missile ai_missiles[MAX_TURRETS];

int ship_setup_size = 0;
float ship_setup_dir = 0;
bool ship_setup_select = false;
bool ship_setup_action = false;
int hover_grid = -1;
GLuint selectBuf[BUFSIZE];
GLint hits;


bool show_fps = true;
bool ai_missiles_done = true;
bool human_missiles_done = true;
bool see_ai_grid = false;

// __        ___       __  __       _       
// \ \      / (_)_ __ |  \/  | __ _(_)_ __  
//  \ \ /\ / /| | '_ \| |\/| |/ _` | | '_ \ 
//   \ V  V / | | | | | |  | | (_| | | | | |
//    \_/\_/  |_|_| |_|_|  |_|\__,_|_|_| |_|
//     the main Windows entry point
/////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	HWND			hwnd;						// window handle
	MSG				msg;						// message
	int				s_frames = 0;				// Frame counter
	bool			done = false;				// flag saying when app is complete
	bool			fullscreen = FULLSCREEN;	// Fullscreen, anyone?
	GLfloat			ship_diffuse[4]   = { 1.0f, 1.0f,  1.0f, 0.0f };
	GLfloat			ship_specular[4]  = { 0.0f, 0.0f,  0.0f, 0.0f };
	GLfloat			light0_position[4]  = { 0.0f, 50.0f,  300.0f, 1.0f };
	GLfloat			light0_diffuse[4]  = { 1.0f, 1.0f,  0.9f, 0.0f };
	GLfloat			light0_ambient[4]  = { 0.2f, 0.2f,  0.2f, 0.0f };
	GLfloat			light0_specular[4]  = { 0.0f, 0.0f,  0.0f, 0.0f };
	float			fog_color[4] = { 0.9f, 0.9f, 0.9f, 1.0f };

	// Register the Window
	if (!InitRegisterWindowClassEx(hInstance))
		return 0;

	// Create the Window
	if (!InitCreateWindowEx(hwnd, hInstance, fullscreen))
		return 0;

	// Create the Console
	if (CONSOLE_WINDOW) {
		if (!InitConsoleWindow())
			return 0;
	}

	ShowWindow(hwnd, SW_SHOW);						// display the window
	UpdateWindow(hwnd);								// update the window

	// seed the random number generator
	srand((unsigned)time(NULL));

	///////////////
	///  INIT   ///
    ///////////////
	g_state = new GameState;

	// set it to default to menu
	g_state->set(STATE_MENU);

	ship_bow.Init("src\\models\\bow.3ds", "src\\textures\\bow256.bmp");
	ship_1gun_w.Init("src\\models\\1gun-w.3ds", "src\\textures\\1gun256.bmp");
	ship_1gun_s.Init("src\\models\\1gun-s.3ds", "src\\textures\\1gun256.bmp");
	ship_1gun_e.Init("src\\models\\1gun-e.3ds", "src\\textures\\1gun256.bmp");
	ship_1gun_n.Init("src\\models\\1gun-n.3ds", "src\\textures\\1gun256.bmp");
	ship_stern.Init("src\\models\\stern.3ds", "src\\textures\\stern256.bmp");
	missile.Init("src\\models\\missile.3ds", "src\\textures\\missile.bmp");
	missile2.Init("src\\models\\missile.3ds", "src\\textures\\missile2.bmp");
	busted_bow.Init("src\\models\\busted-bow.3ds", "src\\textures\\busted-bow.bmp");
	busted_1gun.Init("src\\models\\busted-1gun.3ds", "src\\textures\\busted-1gun.bmp");
	busted_stern.Init("src\\models\\busted-stern.3ds", "src\\textures\\busted-stern.bmp");


	// Second parameter is for building mip maps or not.  true=build, false=don't)
	texture_sky = new Texture("src\\textures\\sky2.bmp", true);
	texture_land = new Texture("src\\textures\\green.bmp", true);
	texture_water = new Texture("src\\textures\\water2.bmp", true);
	b_target = new Texture("src\\textures\\b_target.tga", false);
	b_surrender = new Texture("src\\textures\\b_surrender.tga", false);
	b_binoculars = new Texture("src\\textures\\b_binoculars.tga", false);
	b_shots = new Texture("src\\textures\\b_shots.tga", false);
	b_hits = new Texture("src\\textures\\b_hits.tga", false);
	b_fire = new Texture("src\\textures\\b_fire.tga", false);
	binoculars = new Texture("src\\textures\\binoculars.tga", false);
	s_done = new Texture("src\\textures\\s_done.tga", false);


	test_font.init("Arial", 3, 0.1f, g_HDC);

	g_timer.Init();				// Initialize the timer

	glShadeModel(GL_SMOOTH);
	//glEnable(GL_POLYGON_SMOOTH);
	//glEnable(GL_LINE_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHT0);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, ship_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, ship_specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// This is for putting images into the screen buffer.
	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

	// Clear color.  clear alpha too!!
	glClearColor(0, 0, 0, 0);

	BuildLand(MAP_X, MAP_Z, HEIGHT_MAP_FILE);

	glHint(GL_FOG_HINT, GL_NICEST);
	glFogf(GL_FOG_START, 1.0f);
	glFogf(GL_FOG_END, 500.0f);
	//glFogf(GL_FOG_MODE, GL_LINEAR);
	glFogf(GL_FOG_MODE, GL_EXP);
	//glFogf(GL_FOG_MODE, GL_EXP2);
	glFogf(GL_FOG_DENSITY, 0.002f);
	glFogfv(GL_FOG_COLOR, fog_color);
	glEnable(GL_FOG);

	///////////////////
	///  MAIN Loop  ///
    ///////////////////
	while (!done)
	{
		PeekMessage(&msg, hwnd, NULL, NULL, PM_REMOVE);

		if (msg.message == WM_QUIT)		// did we receive a WM_QUIT message?
		{
			done = true;				// if so, time to quit the application.
		}
		else
		{
	
			// clear screen and depth buffer
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glLoadIdentity();		// reset modelview matrix

			switch(g_state->get())
			{
				case STATE_MENU:		// We're in the menu
					//cout << "entering case STATE_MENU" << endl;
					// 1. evaluate input for this mode
					// 2. 
					glDisable(GL_FOG);
					DrawMenu();		
					break;
				case STATE_SETUP:		// We're setting up the board
					//cout << "entering case STATE_SETUP" << endl;
					glDisable(GL_FOG);
					ShipPlacement();
					HeadsUpDisplay();
					break;
				case STATE_BATTLE:		// We're freely looking around in the command room.
					//cout << "entering case STATE_BATTLE" << endl;
					glEnable(GL_FOG);
					battleMode();
					HeadsUpDisplay();
					if (see_ai_grid)
						DrawAiGrid();
					//cout << "leaving case STATE_BATTLE" << endl;
					break;
				case STATE_TARGETING:	// We're choosing targets
					//cout << "entering case STATE_TARGETING" << endl;
					battleMode();
					targetingMode();
					HeadsUpDisplay();
					if (see_ai_grid)
						DrawAiGrid();
					//DrawShotsRemaining();
					break;
				case STATE_FIRING:		// We're watching our missiles fly!
					glEnable(GL_FOG);
					battleMode();
					AnimateHumanMissiles();
					AnimateAiMissiles();
					HeadsUpDisplay();
					if (see_ai_grid)
						DrawAiGrid();
					if (ai_missiles_done && human_missiles_done)
					{
						g_state->resetAiShotsRemaining();
						g_state->resetHumanShotsRemaining();
						g_state->set(STATE_BATTLE);
					}
					//DrawShotsRemaining();
					break;
				case STATE_FIRING_AND_BINOCULARS:		// We're watching our missiles fly (with binoculars!)
					glEnable(GL_FOG);
					battleMode();
					AnimateHumanMissiles();
					AnimateAiMissiles();
					binocularMode();
					if (ai_missiles_done && human_missiles_done)
					{
						g_state->resetAiShotsRemaining();
						g_state->resetHumanShotsRemaining();
						g_state->set(STATE_BINOCULARS);
					}
					break;
				case STATE_BINOCULARS:	// We're using the binoculars
					glEnable(GL_FOG);
					battleMode();
					binocularMode();
					//cout << "entering case STATE_BINOCULARS" << endl;
					break;
				case STATE_CREDITS:		// We're looking at the credits
					//cout << "entering case STATE_CREDITS" << endl;
					break;
				case STATE_STATISTICS:	// We're looking at the statistics
					//cout << "entering case STATE_STATISTICS" << endl;
					break;
				default:
					//cout << "entering default case" << endl;
					break;
			}

			glFlush();
			SwapBuffers(g_HDC);				// bring back buffer to foreground
			TranslateMessage(&msg);			// translate/dispatch to event queue
			DispatchMessage(&msg);
		}
	}

	if (fullscreen) {
		ChangeDisplaySettings(NULL,0);		// If So Switch Back To The Desktop
		ShowCursor(TRUE);					// Show Mouse Pointer
	}
	return msg.wParam;
}


//   ___       _   _             __  __           _      
//  / _ \ _ __| |_| |__   ___   |  \/  | ___   __| | ___ 
// | | | | '__| __| '_ \ / _ \  | |\/| |/ _ \ / _` |/ _ \
// | |_| | |  | |_| | | | (_) | | |  | | (_) | (_| |  __/
//  \___/|_|   \__|_| |_|\___/  |_|  |_|\___/ \__,_|\___|
//	This function changes our projection mode from 3D to 2D
////////////////////////////////////////////////////////////////
void OrthoMode(int left, int top, int right, int bottom)
{
	// This function takes in rectangle coordinates for our 2D view port.
	// What happens below is, we need to change our projection matrix.  To do this,
	// we call glMatrixMode(GL_PROJECTION) to tell OpenGL the current mode we want.
	// Then we push on a new matrix so we can just call glPopMatrix() to return us
	// to our previous projection matrix, but first we will render in ortho mode.
	//  Next, we load a new identity matrix so that everything is initialize before
	// we go into ortho mode.  One we switch to ortho by calling glOrth(), we then
	// want to set our matrix mode to GL_MODELVIEW, which puts us back and ready to
	// render our world using the model matrix.  We also initialize this new model view
	// matrix for our 2D rendering before drawing anything.

	// Switch to our projection matrix so that we can change it to ortho mode, not perspective.
	glMatrixMode(GL_PROJECTION);						

	// Push on a new matrix so that we can just pop it off to go back to perspective mode
	glPushMatrix();									
	
	// Reset the current matrix to our identify matrix
	glLoadIdentity();								

	//Pass in our 2D ortho screen coordinates.like so (left, right, bottom, top).  The last
	// 2 parameters are the near and far planes.
	glOrtho( 0, 799, 0, 599, 0, 1 );	
	
	// Switch to model view so that we can render the scope image
	glMatrixMode(GL_MODELVIEW);								

	// Initialize the current model view matrix with the identity matrix
	glLoadIdentity();										
}


//  ____                               _   _             __  __           _      
// |  _ \ ___ _ __ ___ _ __   ___  ___| |_(_)_   _____  |  \/  | ___   __| | ___ 
// | |_) / _ \ '__/ __| '_ \ / _ \/ __| __| \ \ / / _ \ | |\/| |/ _ \ / _` |/ _ \
// |  __/  __/ |  \__ \ |_) |  __/ (__| |_| |\ V /  __/ | |  | | (_) | (_| |  __/
// |_|   \___|_|  |___/ .__/ \___|\___|\__|_| \_/ \___| |_|  |_|\___/ \__,_|\___|
//                    |_|                                                        
//	 This function changes our returns our projection mode from 2D to 3D
///////////////////////////////////////////////////////////////////////////////////
void PerspectiveMode()										// Set Up A Perspective View
{
	// This function doesn't really go to perspective mode, since we aren't
	// using gluPerspective(), but if you look in OrthoMode(), we push on
	// a matrix once we change to our projection matrix.  All we need to do is
	// pop that matrix off of our stack and it will return to our original perspective
	// matrix.  So, before we pop the current matrix, we need to tell OpenGL which
	// matrix we want to use by changing our current matrix mode to perspective.
	// Then we can go back to our model view matrix and render normally.

	// Enter into our projection matrix mode
	glMatrixMode( GL_PROJECTION );							

	// Pop off the last matrix pushed on when in projection mode (Get rid of ortho mode)
	glPopMatrix();											

	// Go back to our model view matrix like normal
	glMatrixMode( GL_MODELVIEW );							

	// We should be in the normal 3D perspective mode now
}


// __        ___       ____                 
// \ \      / (_)_ __ |  _ \ _ __ ___   ___ 
//  \ \ /\ / /| | '_ \| |_) | '__/ _ \ / __|
//   \ V  V / | | | | |  __/| | | (_) | (__ 
//    \_/\_/  |_|_| |_|_|   |_|  \___/ \___|
// the Windows Procedure Event Handler
///////////////////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HGLRC hRC;						// rendering context
	static HDC hDC;							// device context
	static bool cheating = false;			// hmm...
	int width, height;						// window width and height

	// Mouse button variables
	static somepoint mouse_point0;
	somevector mouse_vec;

	switch(message)
	{
		// window is being created
		case WM_CREATE:

			hDC = GetDC(hwnd);					// get current window's device context
			g_HDC = hDC;
			InitSetupPixelFormat(hDC);			// call your pixel format setup function

			// create rendering context and make it current
			hRC = wglCreateContext(hDC);
			wglMakeCurrent(hDC, hRC);

			return 0;
			break;


		// window is closing
		case WM_CLOSE:

			// deselect rendering context and delete it
			wglMakeCurrent(hDC, NULL);
			wglDeleteContext(hRC);

			// send WM_QUIT to message queue
			PostQuitMessage(0);
			
			return 0;
			break;


		// window is resizing
		case WM_SIZE:						
			height = HIWORD(lParam);		// retrieve width and height
			width = LOWORD(lParam);

			if ( height == 0 )				// don't want to divide by zero
				height=1;

			// reset the viewport to new dimensions
			glViewport(0, 0, width, height);
			glMatrixMode(GL_PROJECTION);	// set projection matrix
			glLoadIdentity();				// reset projection matrix

			// calculate aspect ratio of window
			gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 2000.0f);

			// Move the camera back in the Z direction
			// this line kept us from pivoting at the damn
			// camera eye's coordinates...gggrrrr. :)
			//
			//glTranslatef(0.0f, 0.0f, zoom_total);

			glMatrixMode(GL_MODELVIEW);		// set modelview matrix
			glLoadIdentity();				// reset modelview matrix

			return 0;
			break;


		//// When left button is released Reset matrix1 to identity matrix if someeone
		//// is still dragging the mouse when they let go of the button.
		//case WM_LBUTTONUP:
		//	glPushMatrix();
		//		glLoadIdentity();
		//		glGetFloatv(GL_MODELVIEW_MATRIX, matrix1);
		//	glPopMatrix();
		//	break;


		case WM_LBUTTONDOWN:

			tmpx = LOWORD(lParam);
			tmpy = HIWORD(lParam);

			switch (g_state->get())
			{
				case STATE_BATTLE:
					cout << "CLICK! (" << tmpx << ", " << tmpy << ")" << endl;
					// Check if quitting...
					if (IsInBox(tmpx, tmpy, 575, 540, 675, 600))
					{
						// deselect rendering context and delete it
						wglMakeCurrent(hDC, NULL);
						wglDeleteContext(hRC);
						// send WM_QUIT to message queue
						PostQuitMessage(0);
					}
					// Check if Entering Targeting mode...
					if (IsInBox(tmpx, tmpy, 125, 540, 225, 600))
					{
						g_state->set(STATE_TARGETING);
						num_human_targets = 0;
					}
					if (IsInBox(tmpx, tmpy, 350, 540, 450, 600))
					{
						g_state->set(STATE_BINOCULARS);
						battle_pitch = 0.0f;
						battle_yaw = 0.0f;
					}
					break;
				case STATE_TARGETING:
					if (IsInBox(tmpx, tmpy, 199, 99, 600, 499))
					{
						tmpx = (tmpx-199) / 20;
						tmpy = (tmpy-99) / 20;
						cout << "You selected: (" << tmpx << ", " << tmpy << ")" << endl;
						//myDelay();
						if (g_state->selectAiTarget(tmpx, tmpy))
							num_human_targets++;
					}
					if (IsInBox(tmpx, tmpy, 616, 256, 676, 316))
					{
						ChooseAiTargets();
						InitAiMissiles();
						InitHumanMissiles();
						g_state->set(STATE_FIRING);
					}
					if (IsInBox(tmpx, tmpy, 575, 540, 675, 600))
					{
						// deselect rendering context and delete it
						wglMakeCurrent(hDC, NULL);
						wglDeleteContext(hRC);
						// send WM_QUIT to message queue
						PostQuitMessage(0);
					}

					break;
				case STATE_FIRING:
					// Check if quitting...
					if (IsInBox(tmpx, tmpy, 575, 540, 675, 600))
					{
						// deselect rendering context and delete it
						wglMakeCurrent(hDC, NULL);
						wglDeleteContext(hRC);
						// send WM_QUIT to message queue
						PostQuitMessage(0);
					}
					if (IsInBox(tmpx, tmpy, 350, 540, 450, 600))
					{
						g_state->set(STATE_FIRING_AND_BINOCULARS);
						battle_pitch = 0.0f;
						battle_yaw = 0.0f;
					}
					break;

				case STATE_SETUP:
					if (IsInBox(tmpx, tmpy, 575, 540, 675, 600))
					{
						// deselect rendering context and delete it
						wglMakeCurrent(hDC, NULL);
						wglDeleteContext(hRC);
						// send WM_QUIT to message queue
						PostQuitMessage(0);
					}
					if (IsInBox(tmpx, tmpy, 616, 256, 676, 316))
					{
						if (g_state->allUnitsPlaced())
						{
							g_state->initAiRandBoard();
							g_state->resetHumanShotsRemaining();
							//g_state->initAiBoardFromFile("src\\cheat-board.txt");
							g_state->set(STATE_BATTLE);
						}
					}
					ship_setup_select = true;
					break;
			}
			break;

		// Right mouse button down
		case WM_RBUTTONDOWN:

			tmpx = LOWORD(lParam);
			tmpy = HIWORD(lParam);

			switch (g_state->get())
			{
			case STATE_TARGETING:
				tmpx = (tmpx-199) / 20;
				tmpy = (tmpy-99) / 20;
				cout << "You selected: (" << tmpx << ", " << tmpy << ")" << endl;
				if (g_state->deselectAiTarget(tmpx, tmpy))
					num_human_targets--;
				break;

			case STATE_SETUP:
				if (!ship_setup_action) {
					ship_setup_action = true;
					//cout << "Right mouse button pressed" << endl;
				}
				break;
			}
			break;

		// Right mouse button up
		case WM_RBUTTONUP:
			switch (g_state->get())
			{
			case STATE_SETUP:
				if (ship_setup_action) {
					//cout << "Right mouse button let go" << endl;
					ship_setup_dir += 90.0f;
					ship_setup_action = false;
				}
				break;
			}
			break;


		// Mouse movement, this is when we take the most action.  We are going to want to
		// see if the mouse is pressed while moving.  If so store that difference in change
		// and apply it to a matrix that we can use later in calculations.
		case WM_MOUSEMOVE:

			tmpx = LOWORD(lParam);
			tmpy = HIWORD(lParam);

			//if (tmpx > 774) tmpx = 774;
			//if (tmpy > 574) tmpy = 574;
			//if (tmpx < 0) tmpx = 0;
			//if (tmpy < 0) tmpy = 0;

			// Get the current mouse location.
			mouse_point1.SetCord((float)tmpx, (float)tmpy, 0);
			

			// compute the change in x, y mouse movement.
			mouse_vec.calculate(mouse_point0, mouse_point1);

			switch (g_state->get())
			{
			case STATE_BATTLE:
			case STATE_SETUP:
			case STATE_FIRING:
				if (wParam & MK_LBUTTON)
				{
					if (mouse_vec.Xdir() < 0)
					{
						battle_yaw += LOOK_SPEED;
						if (battle_yaw > (2 * PI)) battle_yaw -= (float)(2 * PI);
					}
					else if (mouse_vec.Xdir() > 0)
					{
						battle_yaw -= LOOK_SPEED;
						if (battle_yaw < 0.0f) battle_yaw += (float)(2 * PI);
					}
				}
				else
				{
					if (wParam & MK_RBUTTON)
					{
						if (mouse_vec.Ydir() > 0)
						{
							battle_pitch += LOOK_SPEED;
							if (battle_pitch > (float)PI) battle_pitch = (float)PI;
						}
						else if (mouse_vec.Ydir() < 0)
						{
							battle_pitch -= LOOK_SPEED;
							if (battle_pitch < 0.0f) battle_pitch = 0.0f;
						}
					}
				}
				break;
			case STATE_BINOCULARS:
			case STATE_FIRING_AND_BINOCULARS:
				if (wParam & MK_LBUTTON)
				{
					if (mouse_vec.Xdir() < 0)
					{
						battle_yaw += LOOK_SPEED;
						if (battle_yaw > (float)(PI / 2.0f)) battle_yaw = (float)(PI / 2.0f);
					}
					else if (mouse_vec.Xdir() > 0)
					{
						battle_yaw -= LOOK_SPEED;
						if (battle_yaw < (float)(PI / -2.0f)) battle_yaw = (float)(PI / -2.0f);
					}
				}
				else
				{
					if (wParam & MK_RBUTTON)
					{
						if (mouse_vec.Ydir() > 0)
						{
							battle_pitch += LOOK_SPEED;
							if (battle_pitch > (float)(PI)) battle_pitch = (float)(PI);
						}
						else if (mouse_vec.Ydir() < 0)
						{
							battle_pitch -= LOOK_SPEED;
							if (battle_pitch < 0.0f) battle_pitch = 0.0f;
						}
					}
				}
				break;
			default:
				// If the right mouse button is pushed, just figure out if your dragging
				// up or down to zoom in or out by factor of ZOOM_SPEED.
				if (wParam & MK_RBUTTON) {
					if (mouse_vec.Ydir() > 0) zoom += ZOOM_SPEED;
					if (mouse_vec.Ydir() < 0) zoom -= ZOOM_SPEED;
				}
				break;
			}

			// save positions to static variables for next round.
			mouse_point0 = mouse_point1;

			break;


		//  _                  _                     
		// | | _____ _   _  __| | _____      ___ __  
		// | |/ / _ \ | | |/ _` |/ _ \ \ /\ / / '_ \ 
		// |   <  __/ |_| | (_| | (_) \ V  V /| | | |
		// |_|\_\___|\__, |\__,_|\___/ \_/\_/ |_| |_|
		//          |___/                           
		// They pressed a key!
		case WM_KEYDOWN:
			switch (wParam)
			{
			case 'F':
				if (!cheating)
				{
					enableCheat(g_state, CHEAT_NOSETUP);
					cheating = true;
				}
				break;

			case '1':
				if (g_state->get() != STATE_SETUP) {
					g_state->set(STATE_SETUP);
					g_state->initAiBoardFromFile("src\\board-setup.txt");
					g_state->initHumanBoardFromFile("src\\board-setup.txt");
					cout << "BLANK BOARDS INITIALIZED" << endl;
				}
				break;
			case '2':
				glDisable(GL_FOG);
				break;
			case '3':
				glEnable(GL_FOG);
				break;
			case '4':
				show_fps = false;
				break;
			case '5':
				show_fps = true;
				break;
			case '6':
				see_ai_grid = true;
				break;
			case '7':
				see_ai_grid = false;
				break;
				
			case VK_DOWN:
				if (g_state->get() == STATE_BATTLE || g_state->get() == STATE_FIRING_AND_BINOCULARS || g_state->get() == STATE_FIRING)
				{
					battle_zoom -= ZOOM_SPEED;
					if (battle_zoom < -700.0f)
						battle_zoom = -700.0f;
				}
				if (g_state->get() == STATE_SETUP) {
					if (!ship_setup_action) {
						ship_setup_action = true;
						//cout << "Down key pressed" << endl;
					}
				}

				break;

			case VK_UP:
				if (g_state->get() == STATE_BATTLE || g_state->get() == STATE_FIRING_AND_BINOCULARS || g_state->get() == STATE_FIRING)
				{
					battle_zoom += ZOOM_SPEED;
					if (battle_zoom > 170.0f)
						battle_zoom = 170.0f;
				}
				if (g_state->get() == STATE_SETUP) {
					if (!ship_setup_action) {
						ship_setup_action = true;
						//cout << "Up arrow key pressed" << endl;
					}
				}

				break;

			case VK_END:
				if (g_state->get() == STATE_BATTLE || g_state->get() == STATE_BINOCULARS || g_state->get() == STATE_FIRING_AND_BINOCULARS || g_state->get() == STATE_FIRING)
				{
					eye_height -= EYE_Y_SPEED;
					if (eye_height < -23.5f)
						eye_height = -23.5f;
				}

				break;

			case VK_HOME:
				if (g_state->get() == STATE_BATTLE || g_state->get() == STATE_BINOCULARS || g_state->get() == STATE_FIRING_AND_BINOCULARS || g_state->get() == STATE_FIRING)
				{
					eye_height += EYE_Y_SPEED;
					if (eye_height > 600.0f)
						eye_height = 600.0f;
				}
				break;

			case VK_LEFT:
				if (g_state->get() == STATE_BATTLE || g_state->get() == STATE_FIRING_AND_BINOCULARS || g_state->get() == STATE_FIRING)
					battle_yaw -= LOOK_SPEED;

				if (g_state->get() == STATE_SETUP) {
					if (!ship_setup_action) {
						ship_setup_action = true;
						//cout << "Left key pressed" << endl;
					}
				}
				break;

			case VK_RIGHT:
				if (g_state->get() == STATE_BATTLE || g_state->get() == STATE_FIRING_AND_BINOCULARS || g_state->get() == STATE_FIRING)
					battle_yaw += LOOK_SPEED;

				if (g_state->get() == STATE_SETUP) {
					if (!ship_setup_action) {
						ship_setup_action = true;
						//cout << "Right arrow key pressed" << endl;
					}
				}
				break;

			default:
				if (wParam == VK_ESCAPE)
				{
					switch(g_state->get())
					{
					case STATE_BINOCULARS:
						battle_yaw = 0.0f;
						battle_pitch = 0.0f;
						g_state->set(STATE_BATTLE);
						break;
					case STATE_FIRING_AND_BINOCULARS:
						battle_yaw = 0.0f;
						battle_pitch = 0.0f;
						g_state->set(STATE_FIRING);
						break;
					case STATE_TARGETING:
						g_state->set(STATE_BATTLE);
						break;
					default:
						// deselect rendering context and delete it
						wglMakeCurrent(hDC, NULL);
						wglDeleteContext(hRC);
						// send WM_QUIT to message queue
						PostQuitMessage(0);
						break;
					}
				}
				break;
			}

			break;

		//  _                          
		// | | _____ _   _ _   _ _ __  
		// | |/ / _ \ | | | | | | '_ \ 
		// |   <  __/ |_| | |_| | |_) |
		// |_|\_\___|\__, |\__,_| .__/ 
		//           |___/      |_|    
		//       keys are let go
		////////////////////////////////
		case WM_KEYUP:
			switch (wParam)
			{
			case VK_UP:
				if (g_state->get() == STATE_SETUP) {
					if (ship_setup_action) {
						//cout << "Up key let go" << endl;
						ship_setup_size += 1;
						ship_setup_action = false;
					}
				}
				break;

			case VK_DOWN:
				if (g_state->get() == STATE_SETUP) {
					if (ship_setup_action) {
						//cout << "Down key let go" << endl;
						ship_setup_size -= 1;
						ship_setup_action = false;
					}
				}
				break;

			case VK_LEFT:
				if (g_state->get() == STATE_SETUP) {
					if (ship_setup_action) {
						//cout << "Left arrow let go" << endl;
						ship_setup_dir -= 90.0f;
						ship_setup_action = false;
					}
				}
				break;

			case VK_RIGHT:
				if (g_state->get() == STATE_SETUP) {
					if (ship_setup_action) {
						//cout << "Right arrow let go" << endl;
						ship_setup_dir += 90.0f;
						ship_setup_action = false;
					}
				}
				break;

			}
			break;
	}

	return (DefWindowProc(hwnd, message, wParam, lParam));
}



// _   _                _     _   _       ____  _           _
//| | | | ___  __ _  __| |___| | | |_ __ |  _ \(_)___ _ __ | | __ _ _   _
//| |_| |/ _ \/ _` |/ _` / __| | | | '_ \| | | | / __| '_ \| |/ _` | | | |
//|  _  |  __/ (_| | (_| \__ \ |_| | |_) | |_| | \__ \ |_) | | (_| | |_| |
//|_| |_|\___|\__,_|\__,_|___/\___/| .__/|____/|_|___/ .__/|_|\__,_|\__, |
//                                 |_|               |_|            |___/
////////////////////////////////////////////////////////////////////////////////////////
void HeadsUpDisplay()
{
	 /*
     * Disable stuff that's likely to slow down glDrawPixels.
     * (Omit as much of this as possible, when you know in advance
     * that the OpenGL state will already be set correctly.)
     */

	//glPushAttrib(GL_ALL_ATTRIB_BITS);
		glDisable(GL_ALPHA_TEST);
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_FOG);
		glDisable(GL_LIGHTING);
		glDisable(GL_LOGIC_OP);
		glDisable(GL_STENCIL_TEST);
		glDisable(GL_TEXTURE_1D);
		glDisable(GL_TEXTURE_2D);
		glPixelTransferi(GL_MAP_COLOR, GL_FALSE);
		glPixelTransferi(GL_RED_SCALE, 1);
		glPixelTransferi(GL_RED_BIAS, 0);
		glPixelTransferi(GL_GREEN_SCALE, 1);
		glPixelTransferi(GL_GREEN_BIAS, 0);
		glPixelTransferi(GL_BLUE_SCALE, 1);
		glPixelTransferi(GL_BLUE_BIAS, 0);
		glPixelTransferi(GL_ALPHA_SCALE, 1);
		glPixelTransferi(GL_ALPHA_BIAS, 0);

		// Go into 2D mode
		OrthoMode(0, 0, 800, 600);


		//// show targeting button
		//glRasterPos2i(125, 0);
		//glDrawPixels(b_target->GetWidth(), b_target->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_target->GetData());

		//// show binocular button
		//glRasterPos2i(350, 0);
		//glDrawPixels(b_binoculars->GetWidth(), b_binoculars->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_binoculars->GetData());

		// show surrender button
		glRasterPos2i(575, 0);
		glDrawPixels(b_surrender->GetWidth(), b_surrender->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_surrender->GetData());

		if (g_state->get() == STATE_BATTLE)
		{
			// show targeting button
			glRasterPos2i(125, 0);
			glDrawPixels(b_target->GetWidth(), b_target->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_target->GetData());

			// show hit staticstics
			glRasterPos2i(200, 540);
			glDrawPixels(b_hits->GetWidth(), b_hits->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_hits->GetData());
		}

		if (g_state->get() == STATE_BATTLE || g_state->get() == STATE_FIRING)
		{
			// show binocular button
			glRasterPos2i(350, 0);
			glDrawPixels(b_binoculars->GetWidth(), b_binoculars->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_binoculars->GetData());
		}

		if (g_state->get() == STATE_TARGETING)
		{
			// show available shots
			glRasterPos2i(500, 540);
			glDrawPixels(b_shots->GetWidth(), b_shots->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_shots->GetData());

			// show Fire Button
			glRasterPos2i(620, 270);
			glDrawPixels(b_fire->GetWidth(), b_fire->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_fire->GetData());
		}

		if (g_state->get() == STATE_SETUP  && g_state->allUnitsPlaced())
		{
			// show done button
			glRasterPos2i(620, 270);
			glDrawPixels(s_done->GetWidth(), s_done->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, s_done->GetData());
			//glDrawPixels(b_fire->GetWidth(), b_fire->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, b_fire->GetData());
		}


	// Back to 3D mode.
	PerspectiveMode();

	// popping old attribs on again
	//glPopAttrib();

    glEnable(GL_DEPTH_TEST);
    //glEnable(GL_FOG);
    glEnable(GL_LIGHTING);
    glEnable(GL_LOGIC_OP);
    glEnable(GL_STENCIL_TEST);
    glEnable(GL_TEXTURE_1D);
    glEnable(GL_TEXTURE_2D);

	if (g_state->get() == STATE_BATTLE)
		DrawTotalHits();

	if (g_state->get() == STATE_TARGETING)
		DrawShotsRemaining();

}




//  _                       _   _                                   _
// | |_ __ _ _ __ __ _  ___| |_(_)_ __   __ _   _ __ ___   ___   __| | ___
// | __/ _` | '__/ _` |/ _ \ __| | '_ \ / _` | | '_ ` _ \ / _ \ / _` |/ _ \
// | || (_| | | | (_| |  __/ |_| | | | | (_| | | | | | | | (_) | (_| |  __/
//  \__\__,_|_|  \__, |\___|\__|_|_| |_|\__, | |_| |_| |_|\___/ \__,_|\___|
//               |___/                  |___/
////////////////////////////////////////////////////////////////////////////
void targetingMode()
{
	int i,j;
	b_coord * b_ptr;
	short t_hist;
	float alpha = 0.5f;
	short n_shots;

	n_shots = g_state->getHumanShotsRemaining();
	//cout << "Shots remaining: " << n_shots << endl;
	glDisable(GL_LIGHTING);
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	glDepthMask(GL_FALSE);

	// Go into 2D mode
	OrthoMode(0, 0, 800, 600);

	glBegin(GL_QUADS);
		glColor4f(0.0f, 0.0f, 0.0f, alpha);
		glVertex2f(0.0f, 0.0f);
		glVertex2f(799.0f, 0.0f);
		glVertex2f(799.0f, 599.0f);
		glVertex2f(0.0f, 599.0f);
	glEnd();

	glBegin(GL_QUADS);
	for(j=0; j < 20; j++)
	{
		for(i=0; i < 20; i++)
		{
			b_ptr = g_state->getAiCoord(i, j);
			t_hist = g_state->getHumanTargetHistory(i, j);
			if (b_ptr->type == GRID_LAND)
				glColor4f(1.0f, 1.0f, 0.0f, alpha);
			else
			{
				switch (t_hist)
				{
				case TARGET_AVAILABLE:
					glColor4f(0.0f, 0.0f, 1.0f, alpha);
					break;
				case TARGET_SELECTED:
					glColor4f(0.0f, 1.0f, 0.0f, alpha);
					break;
				case TARGET_TRIED:
					glColor4f(0.7f, 0.7f, 0.7f, alpha);
					break;
				case TARGET_HIT:
					glColor4f(1.0f, 0.0f, 0.0f, alpha);
					break;
				default:
					glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
					break;
				}
			}
			glNormal3f(0.0f, 0.0f, 1.0f);
			glVertex2f((float) 20 * (i+1) + 199, (float) 400 - 20 * (j+1) + 99); 
			glVertex2f((float) 20 * (i+1) + 199, (float) 400 - 20 * j + 99); 
			glVertex2f((float) 20 * i + 199,     (float) 400 - 20 * j + 99); 
			glVertex2f((float) 20 * i + 199,     (float) 400 - 20 * (j+1) + 99); 
		}
	}
	glEnd();
	glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
	for(j=0; j < 20; j++)
	{
		for(i=0; i < 20; i++)
		{
			glBegin(GL_LINE_STRIP);
				glVertex2f((float) 20 * (i+1) + 199, (float) 400 - 20 * (j+1) + 99); 
				glVertex2f((float) 20 * (i+1) + 199, (float) 400 - 20 * j + 99); 
				glVertex2f((float) 20 * i + 199,     (float) 400 - 20 * j + 99); 
				glVertex2f((float) 20 * i + 199,     (float) 400 - 20 * (j+1) + 99); 
				glVertex2f((float) 20 * (i+1) + 199, (float) 400 - 20 * (j+1) + 99); 
			glEnd();
		}
	}

	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	// Back to 3D mode.
	PerspectiveMode();
 }
//  ____                      _    _  ____      _     _
// |  _ \ _ __ __ ___      __/ \  (_)/ ___|_ __(_) __| |
// | | | | '__/ _` \ \ /\ / / _ \ | | |  _| '__| |/ _` |
// | |_| | | | (_| |\ V  V / ___ \| | |_| | |  | | (_| |
// |____/|_|  \__,_| \_/\_/_/   \_\_|\____|_|  |_|\__,_|
//           We want to peek at our enemy! :)
////////////////////////////////////////////////////////////////////////////
void DrawAiGrid()
{
	int i,j;
	b_coord * b_ptr;
	short t_hist;
	float alpha = 0.5f;
//	short n_shots;
	float s = 4.0f;
	float x1 = 718.0f;
	float y1 = 0.0f;
	float w = 80.0f;

	glDisable(GL_LIGHTING);
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	glDepthMask(GL_FALSE);

	// Go into 2D mode
	OrthoMode(0, 0, 800, 600);

	glBegin(GL_QUADS);
	for(j=0; j < 20; j++)
	{
		for(i=0; i < 20; i++)
		{
			b_ptr = g_state->getAiCoord(i, j);
			t_hist = g_state->getAiTargetHistory(i, j);
			switch (b_ptr->type)
			{
			case GRID_LAND:
				glColor4f(1.0f, 1.0f, 0.0f, alpha);
				break;
			case GRID_BOW:
			case GRID_1GUN:
			case GRID_STERN:
				glColor4f(1.0f, 1.0f, 1.0f, alpha);
				break;
			default:
				switch (t_hist)
				{
				case TARGET_AVAILABLE:
					glColor4f(0.0f, 0.0f, 1.0f, alpha);
					break;
				case TARGET_SELECTED:
					glColor4f(0.0f, 1.0f, 0.0f, alpha);
					break;
				case TARGET_TRIED:
					glColor4f(0.3f, 0.3f, 0.3f, alpha);
					break;
				case TARGET_HIT:
					glColor4f(1.0f, 0.0f, 0.0f, alpha);
					break;
				default:
					glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
					break;
				}
				break;
			}
			glNormal3f(0.0f, 0.0f, 1.0f);
			glVertex2f((float) s * (i+1) + x1, (float) w - s * (j+1) + y1); 
			glVertex2f((float) s * (i+1) + x1, (float) w - s * j + y1); 
			glVertex2f((float) s * i + x1,     (float) w - s * j + y1); 
			glVertex2f((float) s * i + x1,     (float) w - s * (j+1) + y1); 
		}
	}
	glEnd();
	glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
	for(j=0; j < 20; j++)
	{
		for(i=0; i < 20; i++)
		{
			glBegin(GL_LINE_STRIP);
				glVertex2f((float) s * (i+1) + x1, (float) w - s * (j+1) + y1); 
				glVertex2f((float) s * (i+1) + x1, (float) w - s * j + y1); 
				glVertex2f((float) s * i + x1,     (float) w - s * j + y1); 
				glVertex2f((float) s * i + x1,     (float) w - s * (j+1) + y1); 
				glVertex2f((float) s * (i+1) + x1, (float) w - s * (j+1) + y1); 
			glEnd();
		}
	}

	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	// Back to 3D mode.
	PerspectiveMode();
 }


//  ____ _                             _    _ _____                    _
// / ___| |__   ___   ___  ___  ___   / \  (_)_   _|_ _ _ __ __ _  ___| |_ ___
//| |   | '_ \ / _ \ / _ \/ __|/ _ \ / _ \ | | | |/ _` | '__/ _` |/ _ \ __/ __|
//| |___| | | | (_) | (_) \__ \  __// ___ \| | | | (_| | | | (_| |  __/ |_\__ \
// \____|_| |_|\___/ \___/|___/\___/_/   \_\_| |_|\__,_|_|  \__, |\___|\__|___/
//                                                          |___/
//         Function to make AI pick things to shoot at.
////////////////////////////////////////////////////////////////////////////
void ChooseAiTargets()
{
	int num_available_targets = 0;
	int n_shots = 0;
	int ai_x, ai_z;
	short t_hist = 0;
	bool found_target;

	for(int i=0; i < BOARD_LENGTH; i++)
	{
		for(int j=0; j < BOARD_WIDTH; j++)
		{
			t_hist = g_state->getAiTargetHistory(j, i);
			if (t_hist == TARGET_AVAILABLE)
				num_available_targets++;
		}
	}

	g_state->resetAiShotsRemaining();
	n_shots = g_state->getAiShotsRemaining();
	num_ai_targets = 0;

	// Here is a good place to pick a smarter picking.
	// If this gets completed to an acceptable level, then
	// we fall through to the the next loop, which picks
	// random spaces.
	// 
	// loop through entire grid; pick hits, and add surrounding
	// squares to target list.  When done, continue to next
	// loop.

	// Just pick random targets.
	while (n_shots != 0 && num_available_targets > 0)
	{
		do {
			found_target = false;
			ai_x = (int)( (20.0 * rand()) / (RAND_MAX + 1.0) );
			ai_z = (int)( (20.0 * rand()) / (RAND_MAX + 1.0) );
			cout << "num_available_targets = " << num_available_targets << endl;
			cout << "AI shots remaining    = " << g_state->getAiShotsRemaining() << endl;
			if (g_state->getAiTargetHistory(ai_x, ai_z) == TARGET_AVAILABLE)
			{
				found_target = true;
				g_state->selectHumanTarget(ai_x, ai_z);
			}
		} while (!found_target);
		n_shots = g_state->getAiShotsRemaining();
		num_available_targets--;
		num_ai_targets++;
	}
}

//  ____        _ _     _ _                    _
// | __ ) _   _(_) | __| | |    __ _ _ __   __| |
// |  _ \| | | | | |/ _` | |   / _` | '_ \ / _` |
// | |_) | |_| | | | (_| | |__| (_| | | | | (_| |
// |____/ \__,_|_|_|\__,_|_____\__,_|_| |_|\__,_|
///////////////////////////////////////////////////////////////////////////
void BuildLand(int map_x, int map_z, char * filename)
{
	// texture information
	BITMAPINFOHEADER	g_bitmapInfoHeader;		// temp bitmap info header
	unsigned char		* g_imageData;			// the map image data
	somepoint			p1, p2, p3;				// points used to calculate a vector
	somevector			n;						// normal vector

	g_imageData = LoadBitmapFile(filename, &g_bitmapInfoHeader);
	//g_imageData = LoadBitmapFile("img\\Terrain2.bmp", &g_bitmapInfoHeader);
	
	if (g_imageData)
		cout << "THE LAND IMAGE IS LOADED" << endl;

	// loop through all of the heightfield points, calculating
  	// the coordinates for each point, and calculating the normals
	// for each quad.
  	for (int z = 0; z < map_z; z++)
	{
  		for (int x = 0; x < map_x; x++)
  		{
  			g_terrain[x + map_x * z][0] = float(x) * MAP_SCALE;
  			//g_terrain[x + map_x * z][1] = (float) g_imageData[(z * map_x + x) * 3] / 1.9f; // scaled down by 5.0
  			g_terrain[x + map_x * z][1] = (float)pow(g_imageData[(z * map_x + x) * 3], 2.0) / 200.0f; // scaled down by 5.0
  			g_terrain[x + map_x * z][2] = -float(z) * MAP_SCALE;
			//cout << "(" << g_terrain[x + MAP_X * z][0] << ", " << g_terrain[x + MAP_X * z][1] << ", "
			//	<< g_terrain[x + MAP_X * z][2] << ") - ";

			g_normalData[x + map_x * z][0] = 0.0f;
			g_normalData[x + map_x * z][1] = 1.0f;
			g_normalData[x + map_x * z][2] = 0.0f;

			// Calculate Normal Vectors of faces.
			if (x != 0 && z != 0)
			{
				p1.SetCord(g_terrain[x + map_x * z - 1][0], g_terrain[x + map_x * z - 1][1], g_terrain[x + map_x * z - 1][2]);
				p2.SetCord(g_terrain[x + map_x * z][0], g_terrain[x + map_x * z][1], g_terrain[x + map_x * z][2]);
				p3.SetCord(g_terrain[x + map_x * (z - 1)][0], g_terrain[x + map_x * (z - 1)][1], g_terrain[x + map_x * (z - 1)][2]);
				n.CrossProduct(p2, p1, p3);
				n.normalize();
				g_normalFaceData[x + map_x * (z - 1) - 1][0] = n.Xdir();
				g_normalFaceData[x + map_x * (z - 1) - 1][1] = n.Ydir();
				g_normalFaceData[x + map_x * (z - 1) - 1][2] = n.Zdir();
			}
		}
  	}

	// loop through all of the inner points, and calculating
	// their normals.
  	for (int z = 1; z < (map_z - 1); z++)
	{
  		for (int x = 1; x < (map_x - 1); x++)
  		{
			n.calculate(
				(
					g_normalFaceData[x + map_x * (z - 1) - 1][0] + 
					g_normalFaceData[x + map_x * (z - 1)    ][0] + 
					g_normalFaceData[x + map_x * (z    ) - 1][0] +
					g_normalFaceData[x + map_x * (z    )    ][0]
				),
				(
					g_normalFaceData[x + map_x * (z - 1) - 1][1] + 
					g_normalFaceData[x + map_x * (z - 1)    ][1] + 
					g_normalFaceData[x + map_x * (z    ) - 1][1] +
					g_normalFaceData[x + map_x * (z    )    ][1]
				),
				(
					g_normalFaceData[x + map_x * (z - 1) - 1][2] + 
					g_normalFaceData[x + map_x * (z - 1)    ][2] + 
					g_normalFaceData[x + map_x * (z    ) - 1][2] +
					g_normalFaceData[x + map_x * (z    )    ][0]
				)
			);
			n.normalize();
			g_normalData[x + map_x * (z - 1) - 1][0] = n.Xdir();
			g_normalData[x + map_x * (z - 1) - 1][1] = n.Ydir();
			g_normalData[x + map_x * (z - 1) - 1][2] = n.Zdir();

 		}
  	}

	// used to track current entry in the index array
	int index = 0;
	int currentVertex;

	// loop over all vertices in the terrain map
	for (int z = 0; z < map_z; z++)
	{
		for (int x = 0; x < map_x; x++)
		{
			// vertices are numbered left to right, top to bottom
			currentVertex = z * map_x + x;

			// set the values in the color array      
			//g_colorArray[currentVertex][0] = g_colorArray[currentVertex][1] = 
			//	g_colorArray[currentVertex][2] = g_terrain[x + map_x * z][1]/255.0f;
			g_colorArray[currentVertex][0] = g_colorArray[currentVertex][1] = 
				g_colorArray[currentVertex][2] = g_terrain[x + map_x * z][1]/512.0f + 0.5f;

			// set the values in the texture coordinate array. since the texture      
			// is tiled over each "square", we can use texture wrapping
			g_texcoordArray[currentVertex][0] = (float) x / MAP_TEXTURE_SCALE;
			g_texcoordArray[currentVertex][1] = (float) z / MAP_TEXTURE_SCALE;
		}
	}
	for (int z = 0; z < map_z - 1; z++)
	{
		for (int x = 0; x < map_x; x++)
		{
			currentVertex = z * map_x + x;
			g_indexArray[index++] = currentVertex + map_x;
			g_indexArray[index++] = currentVertex;
		}
	}

	// enable the vertex arrays being used  
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);

	// pass the pointers to OpenGL
	glVertexPointer(3, GL_FLOAT, 0, g_terrain);
	glColorPointer(3, GL_FLOAT, 0, g_colorArray);
	glTexCoordPointer(2, GL_FLOAT, 0, g_texcoordArray);
	glNormalPointer(GL_FLOAT, 0, g_normalData);

	glColor3f(1.0f, 1.0f, 1.0f);

	landCallList = glGenLists(1);
	glNewList(landCallList, GL_COMPILE);

	// Draw terrain
	//
	glPushMatrix();
		//glTranslatef(0.0f, -6.0f, 178.0f); // for 20x20 and 180x180
		glTranslatef(-710.0f, -6.0f, 698.0f); // for 1600x700 and 800 x 350

		// loop through all the triangle strips
		for (int z = 0; z < MAP_Z-1; z++)
		{
	  		// draw the triangles in this strip
			glDrawElements(GL_TRIANGLE_STRIP, MAP_X * 2, GL_UNSIGNED_INT, &g_indexArray[z * MAP_X * 2]);
		}
	glPopMatrix();
	glEndList();

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

}

//  _     _                        _            __  __           _
// | |__ (_)_ __   ___   ___ _   _| | __ _ _ __|  \/  | ___   __| | ___
// | '_ \| | '_ \ / _ \ / __| | | | |/ _` | '__| |\/| |/ _ \ / _` |/ _ \
// | |_) | | | | | (_) | (__| |_| | | (_| | |  | |  | | (_) | (_| |  __/
// |_.__/|_|_| |_|\___/ \___|\__,_|_|\__,_|_|  |_|  |_|\___/ \__,_|\___|
//                    Function to show binocular-masked view
/////////////////////////////////////////////////////////////////////////////
void binocularMode()
{
	 /*
     * Disable stuff that's likely to slow down glDrawPixels.
     * (Omit as much of this as possible, when you know in advance
     * that the OpenGL state will already be set correctly.)
     */
    glDisable(GL_ALPHA_TEST);
    glDisable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_FOG);
    glDisable(GL_LIGHTING);
    glDisable(GL_LOGIC_OP);
    glDisable(GL_STENCIL_TEST);
    glDisable(GL_TEXTURE_1D);
    glDisable(GL_TEXTURE_2D);
    glPixelTransferi(GL_MAP_COLOR, GL_FALSE);
    glPixelTransferi(GL_RED_SCALE, 1);
    glPixelTransferi(GL_RED_BIAS, 0);
    glPixelTransferi(GL_GREEN_SCALE, 1);
    glPixelTransferi(GL_GREEN_BIAS, 0);
    glPixelTransferi(GL_BLUE_SCALE, 1);
    glPixelTransferi(GL_BLUE_BIAS, 0);
    glPixelTransferi(GL_ALPHA_SCALE, 1);
    glPixelTransferi(GL_ALPHA_BIAS, 0);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// Go into 2D mode
	OrthoMode(0, 0, 800, 600);

	// show targeting button
	glRasterPos2i(0, 0);
	glDrawPixels(binoculars->GetWidth(), binoculars->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, binoculars->GetData());

	// Back to 3D mode.
	PerspectiveMode();
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_FOG);
    glEnable(GL_LIGHTING);
    glEnable(GL_LOGIC_OP);
    glEnable(GL_STENCIL_TEST);
    glEnable(GL_TEXTURE_1D);
    glEnable(GL_TEXTURE_2D);
	//	glDisable(GL_BLEND);
}

//  _           _   _   _                            _
// | |__   __ _| |_| |_| | ___   _ __ ___   ___   __| | ___
// | '_ \ / _` | __| __| |/ _ \ | '_ ` _ \ / _ \ / _` |/ _ \
// | |_) | (_| | |_| |_| |  __/ | | | | | | (_) | (_| |  __/
// |_.__/ \__,_|\__|\__|_|\___| |_| |_| |_|\___/ \__,_|\___|
////////////////////////////////////////////////////////////
void battleMode()
{
	DrawFPS();

	switch(g_state->get())
	{
	case STATE_BATTLE:
	case STATE_FIRING:
	case STATE_TARGETING:
		gluLookAt(
				sin(battle_yaw) * (LOOK_RADIUS - battle_zoom) + LOOK_X,	// eye's x-coordinate
				eye_height,													// eye's y-coordinate
				cos(battle_yaw) * (LOOK_RADIUS - battle_zoom) + LOOK_Z,	// eye's z-coordinate
				LOOK_X, LOOK_Y, LOOK_Z,
				0.0f, 1.0f, 0.0f);
		break;
	case STATE_BINOCULARS:
	case STATE_FIRING_AND_BINOCULARS:
		gluLookAt(
				-2.0f * sin(battle_yaw) * (LOOK_RADIUS) + LOOK_X,	// eye's x-coordinate
				25.0f,																// eye's y-coordinate
				-2.0f * cos(battle_yaw) * (LOOK_RADIUS) + LOOK_Z,	// eye's z-coordinate
				-3.0f * sin(battle_yaw) * (LOOK_RADIUS) + LOOK_X,	// look-at x-coordinate
				100 * cos(battle_pitch) - 75,										// look-at y-coordinate
				-3.0f * cos(battle_yaw) * (LOOK_RADIUS) + LOOK_Z,	// look-at z-coordinate
				0.0f, 1.0f, 0.0f);													// up direction
		break;
	}


	glPushMatrix();
	
		glTranslatef(-90.0f, -25.0f, -212.0f);

		// Draw Water
		DrawWater();

		//DrawGrid();

		DrawTerrain();

		DrawSky();

		DrawShips();

	glPopMatrix();

}
//  ____                      ____      _     _
// |  _ \ _ __ __ ___      __/ ___|_ __(_) __| |
// | | | | '__/ _` \ \ /\ / / |  _| '__| |/ _` |
// | |_| | | | (_| |\ V  V /| |_| | |  | | (_| |
// |____/|_|  \__,_| \_/\_/  \____|_|  |_|\__,_|
//           Function to draw grid on water.
////////////////////////////////////////////////////////////
void DrawGrid()
{
	float alpha = 0.2f;
	b_coord * b_ptr;

	for(int j=0; j < 20; j++)
	{
		for(int i=0; i < 20; i++)
		{
			b_ptr = g_state->getHumanCoord(i, j);

			if (b_ptr->type == GRID_LAND)
			{
				glColor4f(1.0f, 1.0f, 0.0f, alpha);
			}
			else
			{
				glColor4f(0.0f, 0.0f, 1.0f, alpha);
			}

			if (hover_grid != -1 && !g_state->allUnitsPlaced())
			{
				int next = 0;
				bool valid = g_state->isValidPlacement(hover_grid, (int) ship_setup_dir, ship_setup_size + 2, HUMAN);

				for (int x = 0; x < ship_setup_size + 2; x++)
				{
					if (hover_grid + next == (j * 20 + i))
					{
						if (valid)
						{
							glColor4f(1.0f, 1.0f, 0.5f, alpha);
						}
						else
						{
							glColor4f(1.0f, 0.0f, 0.0f, 1.0);
						}
					}

					if (ship_setup_dir == 0)
					{
						next += 20;
					}
					else if (ship_setup_dir == 90)
					{
						next += 1;
					}
					else if (ship_setup_dir == 180)
					{
						next -= 20;
					}
					else if (ship_setup_dir == 270)
					{
						next -= 1;
					}
				}
			}

			glPushName(j * 20 + i);

			glBegin(GL_QUADS);

				glNormal3f(0.0f, 1.0f, 0.0f);
				glTranslatef((float)j*9 + 4.5f, -1.0f, (float)i*9 + 4.5f);

				glVertex3f((float)((i+1)* 9.0f), 0.1f, (float)((j+1) * 9.0f)); 
				glVertex3f((float)((i+1)* 9.0f), 0.1f, (float)(j * 9.0f)); 
				glVertex3f((float)(i * 9.0f),    0.1f, (float)(j * 9.0f)); 
				glVertex3f((float)(i * 9.0f),    0.1f, (float)((j+1) * 9.0f));

			glEnd();

			glPopName();

			glColor3f(0.0f, 0.0f, 0.0f);
			glLineWidth(2.0);

			glBegin(GL_LINES);

				glNormal3f(0.0f, 1.0f, 0.0f);
				glTranslatef((float)j*9 + 4.5f, -1.0f, (float)i*9 + 4.5f);

				glVertex3f((float)((i+1)* 9.0f), 0.11f, (float)((j+1) * 9.0f)); 
				glVertex3f((float)((i+1)* 9.0f), 0.11f, (float)(j * 9.0f)); 

				glVertex3f((float)((i+1)* 9.0f), 0.11f, (float)(j * 9.0f)); 
				glVertex3f((float)(i * 9.0f),    0.11f, (float)(j * 9.0f)); 

				glVertex3f((float)(i * 9.0f),    0.11f, (float)(j * 9.0f)); 
				glVertex3f((float)(i * 9.0f),    0.11f, (float)((j+1) * 9.0f));

				glVertex3f((float)(i * 9.0f),    0.11f, (float)((j+1) * 9.0f));
				glVertex3f((float)((i+1)* 9.0f), 0.11f, (float)((j+1) * 9.0f)); 

			glEnd();

			glLineWidth(1.0);

		}
	}
}



//  ____                     _____                   _       
// |  _ \ _ __ __ ___      _|_   _|__ _ __ _ __ __ _(_)_ __  
// | | | | '__/ _` \ \ /\ / / | |/ _ \ '__| '__/ _` | | '_ \ 
// | |_| | | | (_| |\ V  V /  | |  __/ |  | | | (_| | | | | |
// |____/|_|  \__,_| \_/\_/   |_|\___|_|  |_|  \__,_|_|_| |_|
//                Function to draw terrain
/////////////////////////////////////////////////////////////
void DrawTerrain(void)
{
	glColor3f(1.0f, 1.0f, 1.0f);

	// Draw Land
	texture_land->Bind();
	glCallList(landCallList);

}


//  ____                      ____  _     _           
// |  _ \ _ __ __ ___      __/ ___|| |__ (_)_ __  ___ 
// | | | | '__/ _` \ \ /\ / /\___ \| '_ \| | '_ \/ __|
// | |_| | | | (_| |\ V  V /  ___) | | | | | |_) \__ \
// |____/|_|  \__,_| \_/\_/  |____/|_| |_|_| .__/|___/
//                                        |_|        
//           Function to draw ships
/////////////////////////////////////////////////////
void DrawShips(void)
{
	b_coord *human_board;

	glColor3f(1.0f, 1.0f, 1.0f);
	for (int i = 0; i < BOARD_LENGTH; i++)
	{
		for (int j = 0; j < BOARD_WIDTH; j++)
		{
			human_board = g_state->getHumanCoord(j, i);
			if (human_board->type != GRID_WATER && human_board->type != GRID_LAND)
			{
				switch(human_board->type)
				{
				case GRID_BOW:
					glPushMatrix();
						glTranslatef((float)j*9 + 4.5f, -1.0f, (float)i*9 + 4.5f);
						//cout << "(" << j << "," << i << ") Rendering bow." << endl;
						if (human_board->facing == NORTH) glRotatef(180, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == WEST) glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == EAST) glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
						glScalef(SHIP_SCALE, SHIP_SCALE, SHIP_SCALE);
						//texture_bow->Bind();
						//RenderSquare();
						//RenderModel(ship_bow);
						if (human_board->dead)
							busted_bow.Render();
						else
							ship_bow.Render();
					glPopMatrix();
					break;
				case GRID_1GUN:
					glPushMatrix();
						glTranslatef((float)j*9 + 4.5f, -1.0f, (float)i*9 + 4.5f);
						//cout << "(" << j << "," << i << ") Rendering 1gun." << endl;
						if (human_board->facing == NORTH) glRotatef(180, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == WEST) glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == EAST) glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
						glScalef(SHIP_SCALE, SHIP_SCALE, SHIP_SCALE);
						//texture_1gun->Bind();
						//RenderSquare();
						if (human_board->dead)
							busted_1gun.Render();
						else
						{
							switch(human_board->facing)
							{
							case NORTH:
								//RenderModel(ship_1gun_n);
								ship_1gun_n.Render();
								break;
							case WEST:
								//RenderModel(ship_1gun_w);
								ship_1gun_w.Render();
								break;
							case SOUTH:
								//RenderModel(ship_1gun_s);
								ship_1gun_s.Render();
								break;
							case EAST:
								//RenderModel(ship_1gun_e);
								ship_1gun_e.Render();
								break;
							}
						}
					glPopMatrix();
					break;
				case GRID_2GUN:	// Actually it's 1gun
					glPushMatrix();
						glTranslatef((float)j*9 + 4.5f, -1.0f, (float)i*9 + 4.5f);
						//cout << "(" << j << "," << i << ") Rendering 2gun." << endl;
						if (human_board->facing == NORTH) glRotatef(180, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == WEST) glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == EAST) glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
						glScalef(SHIP_SCALE, SHIP_SCALE, SHIP_SCALE);
						//texture_1gun->Bind();
						//RenderSquare();
						switch(human_board->facing)
						{
						case NORTH:
							//RenderModel(ship_1gun_n);
							ship_1gun_n.Render();
							break;
						case WEST:
							//RenderModel(ship_1gun_w);
							ship_1gun_w.Render();
							break;
						case SOUTH:
							//RenderModel(ship_1gun_s);
							ship_1gun_s.Render();
							break;
						case EAST:
							//RenderModel(ship_1gun_e);
							ship_1gun_e.Render();
							break;
						}
					glPopMatrix();
					break;
				case GRID_STERN:
					glPushMatrix();
						glTranslatef((float)j*9 + 4.5f, -1.0f, (float)i*9 + 4.5f);
						//cout << "(" << j << "," << i << ") Rendering stern." << endl;
						if (human_board->facing == NORTH) glRotatef(180, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == WEST) glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
						if (human_board->facing == EAST) glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
						//glTranslatef(0.0f, 0.0f, 2.0f);
						glScalef(SHIP_SCALE, SHIP_SCALE, SHIP_SCALE);
						//texture_stern->Bind();
						//RenderSquare();
						//RenderModel(ship_stern);
						if (human_board->dead)
							busted_stern.Render();
						else
							ship_stern.Render();
					glPopMatrix();
					break;
				default:
					break;
				}
			}
		}
	}
}


//  ____                     __  __                  
// |  _ \ _ __ __ ___      _|  \/  | ___ _ __  _   _ 
// | | | | '__/ _` \ \ /\ / / |\/| |/ _ \ '_ \| | | |
// | |_| | | | (_| |\ V  V /| |  | |  __/ | | | |_| |
// |____/|_|  \__,_| \_/\_/ |_|  |_|\___|_| |_|\__,_|
// Description: Draw Menu Function
//////////////////////////////////////////////////////
void DrawMenu(void)
{
	static float menu_angle = -15.0f;
	static bool angle_inc;

	glEnable(GL_LIGHTING);

	glPushMatrix();
		//glTranslatef(-90.0f, -25.0f, -212.0f);
		glTranslatef(-90.0f, 90.0f, -250.0f);  // THIS ONE

		//glTranslatef(-90.0f, 90.0f, -50.0f);  // TESTING

		glRotatef(45.0, 1.0f, 0.0f, 0.0f);

	DrawWater();

	DrawTerrain();

	glPopMatrix();

	glPushMatrix();
		glColor3f(0.5f, 0.5f, 1.0f);
		glTranslatef(0.0f, 0.0f, -5.0f);
		//glScalef(1.5f, 1.5f, 1.0f);

		// Generate Texture coordinates...
		texture_water->TexGen(GL_OBJECT_LINEAR);
		// Use this texture
		texture_water->Bind();
		//glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		//glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		// Put text on the screen.
		//test_font.PrintString("HarborWars");
		glPushMatrix();
			glTranslatef(-5*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, 1.0f, 1.0f, 0.0f);
			test_font.PrintString("H");
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-4*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, 1.0f, 0.0f, 1.0f);
			test_font.PrintString("a");
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-3*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, 1.0f, 0.0f, 0.0f);
			test_font.PrintString("r");
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-2*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, 0.0f, 0.0f, 1.0f);
			test_font.PrintString("b");
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-1*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, 0.0f, 1.0f, 1.0f);
			test_font.PrintString("o");
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.0f, 0.0f, 0.0f);
			glRotatef(menu_angle, 0.0f, 1.0f, 0.0f);
			test_font.PrintString("r");
		glPopMatrix();
		glPushMatrix();
			glTranslatef( 2*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, -1.0f, 0.0f, -1.0f);
			test_font.PrintString("W");
		glPopMatrix();
		glPushMatrix();
			glTranslatef( 3*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, -1.0f, 0.0f, 0.0f);
			test_font.PrintString("a");
		glPopMatrix();
		glPushMatrix();
			glTranslatef( 4*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, 0.0f, 0.0f, -1.0f);
			test_font.PrintString("r");
		glPopMatrix();
		glPushMatrix();
			glTranslatef( 5*TITLE_LETTER_GAPS, 0.0f, 0.0f);
			glRotatef(menu_angle, 0.0f, -1.0f, -1.0f);
			test_font.PrintString("s");
		glPopMatrix();

		if (menu_angle > 15.0)
			angle_inc = false;
		if (menu_angle < -15.0)
			angle_inc = true;
		menu_angle += (angle_inc) ? MENU_ANGLE_STEP : - MENU_ANGLE_STEP;

		texture_water->UnBind();

		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);

		glEnable(GL_COLOR_MATERIAL);

		glScalef(0.25f, 0.25f, 1.0f);

		//glColor3f(1.0f, 0.0f, 0.0f);
		//glTranslatef(0.0f, -1.0f, 0.0f);
		//test_font.PrintString("Press 'F' for Battlemode");

		glColor3f(1.0f, 0.0f, 0.0f);
		glTranslatef(-0.0f, -1.0f, 0.0f);
		test_font.PrintString("Press '1' for One Player");

		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);

	glPopMatrix();

}


//  ___     ___       ____            
// |_ _|___|_ _|_ __ | __ )  _____  __
//  | |/ __|| || '_ \|  _ \ / _ \ \/ /
//  | |\__ \| || | | | |_) | (_) >  < 
// |___|___/___|_| |_|____/ \___/_/\_\
//         Check in Box
///////////////////////////////////////
bool IsInBox(int m_x, int m_y, int x1, int y1, int x2, int y2)
{
	if (m_x > x1 && m_x < x2)
	{
		if (m_y > y1 && m_y < y2)
		{
			return true;
		}
	}
	return false;
}


//  ____                    __        __    _            
// |  _ \ _ __ __ ___      _\ \      / /_ _| |_ ___ _ __ 
// | | | | '__/ _` \ \ /\ / /\ \ /\ / / _` | __/ _ \ '__|
// | |_| | | | (_| |\ V  V /  \ V  V / (_| | ||  __/ |   
// |____/|_|  \__,_| \_/\_/    \_/\_/ \__,_|\__\___|_|   
//          Function to draw the water
////////////////////////////////////////////////////////
void DrawWater()
{
	static float tex_angle = 0.0f;
	static bool made_call_list = false;

	tex_angle += 0.03f;

	if (tex_angle > (float)(2*PI)) tex_angle -= (float)(2*PI);

	//glEnable(GL_LIGHTING);
	//glEnable(GL_TEXTURE_2D);

	glColor3f(0.6f, 0.8f, 1.0f);


	// This draws the water...let's use PJ's water texture... :)
	texture_water->Bind();
	glMatrixMode(GL_TEXTURE);
		glTranslatef(0.0f, (float)sin(tex_angle) / 16.0f, 0.0f);
	glMatrixMode(GL_MODELVIEW);

	if (!made_call_list)
	{
		waterCallList = glGenLists(1);
		glNewList(waterCallList, GL_COMPILE);
		glPushMatrix();
			glTranslatef(-800.0f, -0.1f, -800.0f);
			glScalef(WATER_SCALE, WATER_SCALE, WATER_SCALE);
			glNormal3f(0.0f, 1.0f, 0.0f);
			glBegin(GL_QUADS);
			for (int i = 0; i < WATER_SIZE; i++)
			{
				for (int j = 0; j < WATER_SIZE; j++)
				{
					//glTexCoord2f(0.0f, 1.0f);
					glTexCoord2f(j/4.0f, (i+1)/4.0f);
					glVertex3f((float)j, 0.0f,  (float)(i + 1));
					//glTexCoord2f(1.0f, 1.0f);
					glTexCoord2f((j+1)/4.0f, (i+1)/4.0f);
					glVertex3f((float)(j+1), 0.0f,  (float)((i + 1)));
					//glTexCoord2f(1.0f, 0.0f);
					glTexCoord2f((j+1)/4.0f, i/4.0f);
					glVertex3f((float)(j+1), 0.0f, (float)i);
					//glTexCoord2f(0.0f, 0.0f);
					glTexCoord2f(j/4.0f, i/4.0f);
					glVertex3f((float)j, 0.0f, (float)i);
				}
			}
			glEnd();
		glPopMatrix();
		glEndList();
		made_call_list = true;
		cout << "MADE WATER CALL LIST" << endl;
	}
	else
		glCallList(waterCallList);

	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
}


//  ____                      ____  _          
// |  _ \ _ __ __ ___      __/ ___|| | ___   _ 
// | | | | '__/ _` \ \ /\ / /\___ \| |/ / | | |
// | |_| | | | (_| |\ V  V /  ___) |   <| |_| |
// |____/|_|  \__,_| \_/\_/  |____/|_|\_\\__, |
//                                       |___/ 
//           Function to draw sky
/////////////////////////////////////////////////
void DrawSky(void)
{
	static bool made_call_list = false;
	static int SkyCallList = 0;

	if (!made_call_list)
	{
		SkyCallList = glGenLists(1);
		glNewList(SkyCallList, GL_COMPILE);

			GLUquadricObj *Sky = gluNewQuadric();
			glPushMatrix();
				glColor3f(1.0f, 1.0f, 1.0f);
				glTranslatef(100.0f, 0.0f, 100.0f);
				glRotatef(90, 1.0f, 0.0f, 0.0f);
				gluQuadricOrientation(Sky, GLU_INSIDE);
				texture_sky->Bind();
				gluQuadricTexture(Sky, true);
				gluSphere(Sky, 700, 20, 5);
			glPopMatrix();

		glEndList();
		made_call_list = true;
		cout << "MADE SKY CALL LIST" << endl;
	}
	else
		glCallList(SkyCallList);

}




//  ____                     _____ ____  ____  
// |  _ \ _ __ __ ___      _|  ___|  _ \/ ___| 
// | | | | '__/ _` \ \ /\ / / |_  | |_) \___ \ 
// | |_| | | | (_| |\ V  V /|  _| |  __/ ___) |
// |____/|_|  \__,_| \_/\_/ |_|   |_|   |____/ 
//              Function to draw FPS
///////////////////////////////////////////////////
void DrawFPS(void)
{
	glColor3f(1.0f, 1.0f, 1.0f);
	int FPS = 0;
	FPS = (int) g_timer.LockFPS(FRAMERATE);
	//cout << FPS << endl;
	
	if (show_fps)
	{
		glPushMatrix();
			//glTranslatef(-90.0f, -25.0f, -212.0f);
			glTranslatef(6.7f, 5.5f, -15);
	
			// Generate Texture coordinates...
			texture_water->TexGen(GL_OBJECT_LINEAR);
			// Use this texture
			texture_water->Bind();
			// Put text on the screen.
			test_font.PrintString(FPS);

			//glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
			//glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
			glDisable(GL_TEXTURE_GEN_S);
			glDisable(GL_TEXTURE_GEN_T);

		glPopMatrix();
	}
}

//  ____                      ____  _           _       ____                      _       _
// |  _ \ _ __ __ ___      __/ ___|| |__   ___ | |_ ___|  _ \ ___ _ __ ___   __ _(_)_ __ (_)_ __   __ _
// | | | | '__/ _` \ \ /\ / /\___ \| '_ \ / _ \| __/ __| |_) / _ \ '_ ` _ \ / _` | | '_ \| | '_ \ / _` |
// | |_| | | | (_| |\ V  V /  ___) | | | | (_) | |_\__ \  _ <  __/ | | | | | (_| | | | | | | | | | (_| |
// |____/|_|  \__,_| \_/\_/  |____/|_| |_|\___/ \__|___/_| \_\___|_| |_| |_|\__,_|_|_| |_|_|_| |_|\__, |
//                                                                                                |___/
//         Function to draw shots remaining
////////////////////////////////////////////////////////////////////////////////////////////////////////
void DrawShotsRemaining(void)
{
	glColor3f(1.0f, 1.0f, 1.0f);
	short num = 0;

	num = g_state->getHumanShotsRemaining();

	glPushMatrix();
		//glTranslatef(-90.0f, -25.0f, -212.0f);
		glTranslatef(2.4f, 5.5f, -15.25);
	
		// Generate Texture coordinates...
		texture_water->TexGen(GL_OBJECT_LINEAR);
		// Use this texture
		texture_water->Bind();
		// Put text on the screen.
		test_font.PrintString(num);

		//glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		//glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);

	glPopMatrix();
}

//  ____                     _____     _        _ _   _ _ _
// |  _ \ _ __ __ ___      _|_   _|__ | |_ __ _| | | | (_) |_ ___
// | | | | '__/ _` \ \ /\ / / | |/ _ \| __/ _` | | |_| | | __/ __|
// | |_| | | | (_| |\ V  V /  | | (_) | || (_| | |  _  | | |_\__ \
// |____/|_|  \__,_| \_/\_/   |_|\___/ \__\__,_|_|_| |_|_|\__|___/
//         Function to show total hits
/////////////////////////////////////////////////////////////////////
void DrawTotalHits(void)
{
	glColor3f(1.0f, 1.0f, 1.0f);
	short num = 0;

	num = num_human_hits;

	glPushMatrix();
		//glTranslatef(-90.0f, -25.0f, -212.0f);
		glTranslatef(-3.9f, 5.5f, -15.25);
	
		// Generate Texture coordinates...
		texture_water->TexGen(GL_OBJECT_LINEAR);
		// Use this texture
		texture_water->Bind();
		// Put text on the screen.
		test_font.PrintString(num);

		//glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		//glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);

	glPopMatrix();
}

//  ___       _ _      _    _ __  __ _         _ _
// |_ _|_ __ (_) |_   / \  (_)  \/  (_)___ ___(_) | ___  ___
//  | || '_ \| | __| / _ \ | | |\/| | / __/ __| | |/ _ \/ __|
//  | || | | | | |_ / ___ \| | |  | | \__ \__ \ | |  __/\__ \
// |___|_| |_|_|\__/_/   \_\_|_|  |_|_|___/___/_|_|\___||___/
//             Initializes AI's missiles
/////////////////////////////////////////////////////////////////
void InitAiMissiles()
{
	b_coord * temp_coord;
	int num_turrets = 0;
	int num_targets = 0;

	ai_missiles_done = false;

	// Get coordinates of each gun that's going to shoot.
	for(int i=0 ; i < BOARD_LENGTH; i++)
	{
		for(int j=0; j < BOARD_WIDTH; j++)
		{
			temp_coord = g_state->getAiCoord(j, i);
			if (temp_coord->type == GRID_1GUN && (!temp_coord->dead))
			{
				ai_gun[num_turrets].idx = num_turrets;
				ai_gun[num_turrets].x   = (float)j;
				ai_gun[num_turrets++].z = (float)i;
			}
		}
	}

	// create list of targets.
	for (int i=0; i < BOARD_LENGTH; i++)
	{
		for (int j=0; j < BOARD_WIDTH; j++)
		{
			if (g_state->getAiTargetHistory(j, i) == TARGET_SELECTED)
			{
				ai_target_list[num_targets++].SetCord((float)j, 0.0f, (float)i);
			}
		}
	}

	for (int i=0; i < num_targets; i++)
	{
		ai_missiles[i].Init();
		ai_missiles[i].InitTime(g_timer.GetTotalElapsedSeconds(), (float)i / 2.0f);
		ai_missiles[i].SetStartPoint(ai_gun[i].x * 9.0f - 85.5f, -2.0f, ai_gun[i].z * 9.0f - 1100.0f);
		ai_missiles[i].SetEndPoint(ai_target_list[i].getx() * 9.0f - 85.5f, -2.0f, ai_target_list[i].getz() * 9.0f - 212); 
		//ai_missiles[i].SetStartPoint(ai_target_list[i].getx() * 9.0f - 85.5f, -2.0f, ai_target_list[i].getz() * 9.0f - 900);
		//ai_missiles[i].SetEndPoint(ai_gun[i].x * 9.0f - 85.5f, -2.0f, ai_gun[i].z * 9.0f - 210.0f); 
		ai_missiles[i].SetTargetCoord((int)ai_target_list[i].getx(), (int)ai_target_list[i].getz());
		ai_missiles[i].SetApex(MISSILE_APEX); 
		ai_missiles[i].Precalculate();
	}
}

//  ___       _ _   _   _                             __  __ _         _ _
// |_ _|_ __ (_) |_| | | |_   _ _ __ ___   __ _ _ __ |  \/  (_)___ ___(_) | ___  ___
//  | || '_ \| | __| |_| | | | | '_ ` _ \ / _` | '_ \| |\/| | / __/ __| | |/ _ \/ __|
//  | || | | | | |_|  _  | |_| | | | | | | (_| | | | | |  | | \__ \__ \ | |  __/\__ \
// |___|_| |_|_|\__|_| |_|\__,_|_| |_| |_|\__,_|_| |_|_|  |_|_|___/___/_|_|\___||___/
//                            Initializes human player's missiles
////////////////////////////////////////////////////////////////////////////////////////
void InitHumanMissiles()
{
	b_coord * temp_coord;
	int num_turrets = 0;
	int num_targets = 0;

	human_missiles_done = false;

	// Get coordinates of each gun that's going to shoot.
	for(int i=0 ; i < BOARD_LENGTH; i++)
	{
		for(int j=0; j < BOARD_WIDTH; j++)
		{
			temp_coord = g_state->getHumanCoord(j, i);
			if (temp_coord->type == GRID_1GUN && (!temp_coord->dead))
			{
				human_gun[num_turrets].idx = num_turrets;
				human_gun[num_turrets].x   = (float)j;
				human_gun[num_turrets++].z = (float)i;
			}
		}
	}

	// create random list of numbers.
	for (int i=0; i < num_turrets; i++)
	{
		rand_human_turrets[i] = (int)( (1000.0 * rand()) / (RAND_MAX + 1.0) );
		//cout << "Random number " << i << ": " << rand_human_turrets[i] << endl;
	}

	// We take the random list, and sort it.  While doing so, we perform the
	// same operations on our list of turrets, thus randomizing the order we
	// fire from our guns. :)
	RandomizeHumanGuns();


	// create list of targets.
	for (int i=0; i < BOARD_LENGTH; i++)
	{
		for (int j=0; j < BOARD_WIDTH; j++)
		{
			if (g_state->getHumanTargetHistory(j, i) == TARGET_SELECTED)
			{
				human_target_list[num_targets++].SetCord((float)j, 0.0f, (float)i);
			}
		}
	}

	for (int i=0; i < num_targets; i++)
	{
		human_missiles[i].Init();
		human_missiles[i].InitTime(g_timer.GetTotalElapsedSeconds(), (float)i / 2.0f);
		human_missiles[i].SetStartPoint(human_gun[i].x * 9.0f - 85.5f, 0.0f, human_gun[i].z * 9.0f - 211.0f);
		human_missiles[i].SetEndPoint(human_target_list[i].getx() * 9.0f - 85.5f, 0.0f, human_target_list[i].getz() * 9.0f - 1100);
		human_missiles[i].SetTargetCoord((int)human_target_list[i].getx(), (int)human_target_list[i].getz());
		human_missiles[i].SetApex(MISSILE_APEX);
		human_missiles[i].Precalculate();
	}
}

//     _          _                 _          _    _ __  __ _         _ _
//    / \   _ __ (_)_ __ ___   __ _| |_ ___   / \  (_)  \/  (_)___ ___(_) | ___  ___
//   / _ \ | '_ \| | '_ ` _ \ / _` | __/ _ \ / _ \ | | |\/| | / __/ __| | |/ _ \/ __|
//  / ___ \| | | | | | | | | | (_| | ||  __// ___ \| | |  | | \__ \__ \ | |  __/\__ \
// /_/   \_\_| |_|_|_| |_| |_|\__,_|\__\___/_/   \_\_|_|  |_|_|___/___/_|_|\___||___/
//                      Animates the missiles the human shoots.
///////////////////////////////////////////////////////////////////////////////////////
void AnimateAiMissiles()
{
	//glTranslatef(-90.0f, -25.0f, -212.0f);
	int num_live_missiles = 0;
	short t_hist;

	for (int i=0; i < num_ai_targets; i++)
	{
		t_hist = g_state->getAiTargetHistory(ai_missiles[i].GetTargetXCoord(), ai_missiles[i].GetTargetZCoord());
		glPushMatrix();
			switch (ai_missiles[i].TranslateAndRotate(g_timer.GetTotalElapsedSeconds(), 0.0f))
			{
			case MISSILE_FLYING:
				missile2.Render();
				num_live_missiles++;
				break;
			case MISSILE_DEAD:
				if (g_state->getAiTargetHistory(ai_missiles[i].GetTargetXCoord(), ai_missiles[i].GetTargetZCoord()) == TARGET_SELECTED)
				{
					//cout << "AI Firing at (" << ai_missiles[i].GetTargetXCoord() << ", " << ai_missiles[i].GetTargetZCoord() << ")" << endl;

					if (g_state->fireAtHuman(ai_missiles[i].GetTargetXCoord(), ai_missiles[i].GetTargetZCoord()))
					{
						num_ai_hits++;
					}
				}
				break;
			case MISSILE_SLEEPING:
				// do nothing
				break;
			}
		glPopMatrix();

	}
	if (num_live_missiles == 0)
		ai_missiles_done = true;
}

//     _          _                 _       _   _                             __  __ _         _ _
//    / \   _ __ (_)_ __ ___   __ _| |_ ___| | | |_   _ _ __ ___   __ _ _ __ |  \/  (_)___ ___(_) | ___  ___
//   / _ \ | '_ \| | '_ ` _ \ / _` | __/ _ \ |_| | | | | '_ ` _ \ / _` | '_ \| |\/| | / __/ __| | |/ _ \/ __|
//  / ___ \| | | | | | | | | | (_| | ||  __/  _  | |_| | | | | | | (_| | | | | |  | | \__ \__ \ | |  __/\__ \
// /_/   \_\_| |_|_|_| |_| |_|\__,_|\__\___|_| |_|\__,_|_| |_| |_|\__,_|_| |_|_|  |_|_|___/___/_|_|\___||___/
//                      Animates the missiles the human shoots.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AnimateHumanMissiles()
{
	//glTranslatef(-90.0f, -25.0f, -212.0f);
	int num_live_missiles = 0;
	short t_hist;

	for (int i=0; i < num_human_targets; i++)
	{
		t_hist = g_state->getHumanTargetHistory(human_missiles[i].GetTargetXCoord(), human_missiles[i].GetTargetZCoord());
		glPushMatrix();
			//switch (human_missiles[human_gun[i].idx].TranslateAndRotate(g_timer.GetTotalElapsedSeconds()))
			switch (human_missiles[i].TranslateAndRotate(g_timer.GetTotalElapsedSeconds(), 180.0f))
			{
			case MISSILE_FLYING:
				missile.Render();
				num_live_missiles++;
				break;
			case MISSILE_DEAD:
				//cout << "Target (" << human_missiles[i].GetTargetXCoord() << ", " << human_missiles[i].GetTargetZCoord() << ") is ";
				//switch (t_hist)
				//{
				//case TARGET_SELECTED:
				//	cout << "TARGET_SELECTED" << endl;
				//	break;
				//case TARGET_AVAILABLE:
				//	cout << "TARGET_AVAILABLE" << endl;
				//	break;
				//case TARGET_UNAVAILABLE:
				//	cout << "TARGET_UNAVAILABLE" << endl;
				//	break;
				//case TARGET_HIT:
				//	cout << "TARGET_HIT" << endl;
				//	break;
				//case TARGET_TRIED:
				//	cout << "TARGET_TRIED" << endl;
				//	break;
				//}
				if (g_state->getHumanTargetHistory(human_missiles[i].GetTargetXCoord(), human_missiles[i].GetTargetZCoord()) == TARGET_SELECTED)
				{
					//cout << "HUMAN Firing at (" << human_missiles[i].GetTargetXCoord() << ", " << human_missiles[i].GetTargetZCoord() << ")" << endl;

					if (g_state->fireAtAi(human_missiles[i].GetTargetXCoord(), human_missiles[i].GetTargetZCoord()))
						num_human_hits++;
				}
				break;
			case MISSILE_SLEEPING:
				// do nothing
				break;
			}
		glPopMatrix();

	}
	if (num_live_missiles == 0)
		human_missiles_done = true;

}

//  _   _                              ___        _      _                   _
// | | | |_   _ _ __ ___   __ _ _ __  / _ \ _   _(_) ___| | _____  ___  _ __| |_
// | |_| | | | | '_ ` _ \ / _` | '_ \| | | | | | | |/ __| |/ / __|/ _ \| '__| __|
// |  _  | |_| | | | | | | (_| | | | | |_| | |_| | | (__|   <\__ \ (_) | |  | |_
// |_| |_|\__,_|_| |_| |_|\__,_|_| |_|\__\_\\__,_|_|\___|_|\_\___/\___/|_|   \__|
//            Sorts human's array of guns
////////////////////////////////////////////////////////////////////////////////
void HumanQuicksort(int left, int right)
{
	if(left < right) {
		int split_pt = HumanPartition(left, right);
		HumanQuicksort(left, split_pt);
		HumanQuicksort(split_pt+1, right);
		}
}

//  _   _                             ____            _   _ _   _
// | | | |_   _ _ __ ___   __ _ _ __ |  _ \ __ _ _ __| |_(_) |_(_) ___  _ __
// | |_| | | | | '_ ` _ \ / _` | '_ \| |_) / _` | '__| __| | __| |/ _ \| '_ \
// |  _  | |_| | | | | | | (_| | | | |  __/ (_| | |  | |_| | |_| | (_) | | | |
// |_| |_|\__,_|_| |_| |_|\__,_|_| |_|_|   \__,_|_|   \__|_|\__|_|\___/|_| |_|
//            Used by HumanQuicksort to split working-list
///////////////////////////////////////////////////////////////////////////////
int HumanPartition(int left, int right)
{
	int val = rand_human_turrets[left];
	int lm = left-1;
	int rm = right+1;
	for(;;) {
		do
			rm--;
		while (rand_human_turrets[rm] > val);
		
		do 
			lm++;
		while( rand_human_turrets[lm] < val);

		if(lm < rm) {
			int tempr = rand_human_turrets[rm];
			rand_human_turrets[rm] = rand_human_turrets[lm];
			rand_human_turrets[lm] = tempr;
			t_list tempz = human_gun[rm];
			human_gun[rm] = human_gun[lm];
			human_gun[lm] = tempz;
			}
		else 
			return rm;
		}
}


//  ____                 _                 _         _   _                              ____
// |  _ \ __ _ _ __   __| | ___  _ __ ___ (_)_______| | | |_   _ _ __ ___   __ _ _ __  / ___|_   _ _ __  ___
// | |_) / _` | '_ \ / _` |/ _ \| '_ ` _ \| |_  / _ \ |_| | | | | '_ ` _ \ / _` | '_ \| |  _| | | | '_ \/ __|
// |  _ < (_| | | | | (_| | (_) | | | | | | |/ /  __/  _  | |_| | | | | | | (_| | | | | |_| | |_| | | | \__ \
// |_| \_\__,_|_| |_|\__,_|\___/|_| |_| |_|_/___\___|_| |_|\__,_|_| |_| |_|\__,_|_| |_|\____|\__,_|_| |_|___/
//                           Make a random order of which guns shoot first.
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void RandomizeHumanGuns()
{
	HumanQuicksort(0, num_human_targets-1);
}


//  ____  _     _       ____  _                                     _   
// / ___|| |__ (_)_ __ |  _ \| | __ _  ___ ___ _ __ ___   ___ _ __ | |_ 
// \___ \| '_ \| | '_ \| |_) | |/ _` |/ __/ _ \ '_ ` _ \ / _ \ '_ \| __|
//  ___) | | | | | |_) |  __/| | (_| | (_|  __/ | | | | |  __/ | | | |_ 
// |____/|_| |_|_| .__/|_|   |_|\__,_|\___\___|_| |_| |_|\___|_| |_|\__|
//              |_|                                                    
//                    Function to place ships
//////////////////////////////////////////////////////////
void ShipPlacement(void)
{
	GLint viewport[4];
	GLfloat depth;
	GLdouble mvmatrix[16], projmatrix[16];
	GLint realy;  /*  OpenGL y coordinate position  */
	GLdouble wx, wy, wz;  /*  returned world x, y, z coords  */
	//float ratio;

	///
	///  Just in time error checking
	/// 

	// Size DOES matter.. ;)
	if (ship_setup_size < 1)
	{
		ship_setup_size = 1;
	}
	else if (ship_setup_size > 8)
	{
		ship_setup_size = 8;
	}

	// Checking for bad directions
	if (ship_setup_dir < 0)
	{
		ship_setup_dir = ship_setup_dir + 360;
	}
	else if (ship_setup_dir > 270)
	{
		ship_setup_dir = 0;
	}


	/////
	/////  This first part mainly is for PICKING.  We are going to see if the
	/////  mouse is over portions of the grid and use that info to place the
	/////  ships where the user wants.
	/////

	//// set up the select buffer
	//glSelectBuffer(BUFSIZE,selectBuf);

	//// grabbing viewport and current matricies
	//glGetIntegerv(GL_VIEWPORT,viewport);

	//// Going into select Render mode
	//glRenderMode(GL_SELECT);

	//// initializing the names stack (don't know if I should do this every time)
	//glInitNames();

	//// Switch into projection matrix
	//glMatrixMode(GL_PROJECTION);

	//// push matrix to save old one
	//glPushMatrix();
	//	
	//	// clear the matrix
	//	glLoadIdentity();

	//	// grab the PICKs of this projection matrix based on my global
	//	// mouse coordinates tmpx, tmpy.
	//	gluPickMatrix(tmpx, viewport[3] - tmpy, 1, 1, viewport);

	//	// viewport[2] is screen width, viewport[3] is height
	//	ratio = (viewport[2] + 0.0f) / viewport[3];

	//	// this is close to our normal perspective call
	//	gluPerspective(45.0f, ratio, 0.1, 1000);

	//	// Switch to modelview to draw the objects with names
	//	glMatrixMode(GL_MODELVIEW);

	//	// must use the same transformations as below.
	//	glTranslatef(-90.0f, 90.0f, -250.0f);
	//	glRotatef(90.0, 1.0f, 0.0f, 0.0f);

	//	static int oldx;
	//	
	//	// just need to draw the grid
	//	//if (oldx != tmpx)
	//		//DrawGrid();
	//	

	//	oldx = tmpx;
	//	
	//	// switch back into projection
	//	glMatrixMode(GL_PROJECTION);

	//glPopMatrix();

	//// Going back to modelview
	//glMatrixMode(GL_MODELVIEW);
	//
	//// clearing modelview
	//glLoadIdentity();

	//// Lets switch it back to RENDER mode and collect our HIT data
	//hits = glRenderMode(GL_RENDER);

	//// process our hit data if there is such a thing
	//if (hits != 0){
	//	hover_grid = processHits(hits,selectBuf,0);
	//}
	//
	//// if they pressed a button above lets not disappoint.
	//if(ship_setup_select)
	//{
	//	if (hover_grid != -1) {
	//		//cout << "Placing ship at grid: " << hover_grid << endl;
	//		bool valid = g_state->isValidPlacement(hover_grid, (int) ship_setup_dir, ship_setup_size + 2, HUMAN);

	//		if (valid)
	//			g_state->placeShipOnBoard(hover_grid, (int) ship_setup_dir, ship_setup_size + 2, HUMAN);
	//	}
	//	ship_setup_select = false;
	//}

	/////
	/////  End of PICKING section.
	/////  I did it all for the PICKING.... the PICKING... the PICKING...
	/////

	DrawFPS();

	glPushMatrix();
	
		glTranslatef(-90.0f, 90.0f, -250.0f);

		glRotatef(90.0, 1.0f, 0.0f, 0.0f);

		DrawWater();

		DrawGrid();

		DrawTerrain();

		DrawShips();

		// default modifiers for placing ships
		int x_mod = 0;
		int y_mod = 1;
		int direction = SOUTH;

		if (ship_setup_dir == 90) {
			x_mod = 1;
			y_mod = 0;
			direction = EAST;
		}

		if (ship_setup_dir == 180) {
			y_mod = -1;
			direction = NORTH;
		}

		if (ship_setup_dir == 270) {
			x_mod = -1;
			y_mod = 0;
			direction = WEST;
		}

		// save all the matricies to use them in UnProject
		glGetIntegerv(GL_VIEWPORT,viewport);
		glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix); 
		glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);

		// calclate my real y value
		realy = viewport[3] - (GLint) tmpy - 1;

		// need to get the depth value for what is projected under the pointer.
		glReadPixels(tmpx, realy, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &depth);

		//debugging...
		//cout << "Coordinates at cursor are: (" << tmpx << "," << realy << ")" << endl;            
		gluUnProject((GLdouble) tmpx, (GLdouble) realy, depth, mvmatrix, projmatrix, viewport, &wx, &wy, &wz);
		//cout << "World coords at z = 0.0 are (" << wx << "," << wy << "," << wz << ")" << endl;

		// hahahaha... much better than Pick
		hover_grid = (((int)wz / 9) * 20 + ((int)wx / 9));

		// if they pressed a button above lets not disappoint.
		if(ship_setup_select)
		{
			if (hover_grid != -1) {
				//cout << "Placing ship at grid: " << hover_grid << endl;
				bool valid = g_state->isValidPlacement(hover_grid, (int) ship_setup_dir, ship_setup_size + 2, HUMAN);

				if (valid)
					g_state->placeShipOnBoard(hover_grid, (int) ship_setup_dir, ship_setup_size + 2, HUMAN);
			}
			ship_setup_select = false;
		}


		// Now lets draw the ships already!
		if (!g_state->allUnitsPlaced())
		{
			glPushMatrix();
				
				glPushMatrix();
					glTranslatef((float) wx, 0.0f,(float) wz);
					glScalef(SHIP_SCALE, SHIP_SCALE, SHIP_SCALE);
					glRotatef(ship_setup_dir + 180.0f, 0.0f, 1.0f, 0.0f);
					ship_bow.Render();
				glPopMatrix();

				for (int x = 0; x < ship_setup_size; x++) {
					glPushMatrix();
						glTranslatef((float) wx, 0.0f, (float) wz);
						glTranslatef((float) (x_mod * (x + 1)) * 9.0f, 0.0f, (y_mod * (x + 1)) * 9.0f);
						glScalef(SHIP_SCALE, SHIP_SCALE, SHIP_SCALE);
						glRotatef(ship_setup_dir, 0.0f, 1.0f, 0.0f);
						if (direction == NORTH)
						{
							ship_1gun_n.Render();
						}
						else if (direction == SOUTH)
						{
							ship_1gun_s.Render();
						}
						else if (direction == EAST)
						{
							ship_1gun_e.Render();
						}
						else if (direction == WEST)
						{
							ship_1gun_w.Render();
						}

					glPopMatrix();
				}

				glPushMatrix();
					glTranslatef((float) wx, 0.0f, (float) wz);
					glTranslatef((float) (x_mod * 9.0 * (ship_setup_size + 1)), 0.0f, (y_mod * (ship_setup_size + 1)) * 9.0f);
					glScalef(SHIP_SCALE, SHIP_SCALE, SHIP_SCALE);
					glRotatef(ship_setup_dir + 180.0f, 0.0f, 1.0f, 0.0f);
					ship_stern.Render();
				glPopMatrix();

			glPopMatrix();
		}

		glPopMatrix();

}

 
//                  ____       _
//  _ __ ___  _   _|  _ \  ___| | __ _ _   _
// | '_ ` _ \| | | | | | |/ _ \ |/ _` | | | |
// | | | | | | |_| | |_| |  __/ | (_| | |_| |
// |_| |_| |_|\__, |____/ \___|_|\__,_|\__, |
//            |___/                    |___/
//              Function to sleep for a few cycles.
//////////////////////////////////////////////////////
void myDelay()
{
	int i;

	// Do stupid loop
	for(i=0; i < 100000000; i++);
}


int processHits(GLint hits, GLuint buffer[], int sw)
{
	GLint i, j, numberOfNames;
	GLuint names, *ptr, minZ,*ptrNames;
	
	ptr = (GLuint *) buffer;
	minZ = 0xffffffff;
	
	for (i = 0; i < hits; i++) {	
		names = *ptr;
	  	ptr++;
	  	if (*ptr < minZ) {
			numberOfNames = names;
			minZ = *ptr;
			ptrNames = ptr + 2;
		}
		ptr += names+2;
	}
	
	if (numberOfNames > 0)
	{
		ptr = ptrNames;
		
		for (j = 0; j < numberOfNames; j++,ptr++) { 
			return *ptr;
		}
		
	}

	return -1;
}