/*
   image.h
   
   image manipulation library header file.
   
   Copyright (c) 2003 Patrick Lynch <plynch@sphere.org>
   Copyright (c) 2003 Brian Jared <bjared@g3.org>
   
   Compiled from many sources and re-written.
   
   (we should give props to the stolen code)
   
*/

#pragma once
#ifndef __IMAGE_H
#define __IMAGE_H

////// Defines
#define BITMAP_ID 0x4D42		// the universal bitmap ID

////// Includes
#include <stdio.h>
#include <windows.h>
#include <gl/gl.h>

// types of texture file formats
enum imgTypes_t
{
	PCX,
	BMP,
	TGA
};

// For TGA Files
typedef struct 
{
     unsigned char imageTypeCode;
     short int     imageWidth;
     short int     imageHeight;
     unsigned char bitCount;
     unsigned char *imageData;
} TGAFILE;

// only partial pcx file header
typedef struct
{
	unsigned char manufacturer;
	unsigned char version;
	unsigned char encoding;
	unsigned char bits;
	unsigned char xMin;
	unsigned char yMin;
	unsigned char xMax;
	unsigned char yMax;
	unsigned char *palette;
} PCXHEADER;

// BEGIN -- Got this from game-tutorials.com ...
// These defines are used to tell us about the type of TARGA file it is
#define TGA_RGB		 2		// This tells us it's a normal RGB (really BGR) file
#define TGA_A		 3		// This tells us it's a ALPHA file
#define TGA_RLE		10		// This tells us that the targa is Run-Length Encoded (RLE)

// This is our image structure for our targa data
struct tImageTGA
{
	int channels;			// The channels in the image (3 = RGB : 4 = RGBA)
	int sizeX;				// The width of the image in pixels
	int sizeY;				// The height of the image in pixels
	unsigned char *data;	// The image pixel data
};
// END -- Got this from game-tutorials.com ...

void DrawBitmap(long, long, unsigned char *);

unsigned char * LoadBitmapFile(char *, BITMAPINFOHEADER *);

unsigned char *LoadPCXFile(char *, PCXHEADER *);

int WriteBitmapFile(char *, int, int, unsigned char *);

int LoadTGAFile(char *, TGAFILE *);

int WriteTGAFile(char *, short int, short int, unsigned char *);

tImageTGA *LoadTGA(const char *, HWND);

#endif