/*
   init.cpp
   
   initialization functions
   
   Copyright (c) 2003 Patrick Lynch <plynch@sphere.org>
   Copyright (c) 2003 Brian Jared <bjared@g3.org>
   
   Compiled from many sources and re-written.
   
   (we should give props to the stolen code)
   
*/

#include "init.h"

extern LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

// function to set the pixel format for the device context
void InitSetupPixelFormat(HDC hDC)
{
	int nPixelFormat;					// our pixel format index

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),	// size of structure
		1,								// default version
		PFD_DRAW_TO_WINDOW |			// window drawing support
		PFD_SUPPORT_OPENGL |			// OpenGL support
		PFD_DOUBLEBUFFER,				// double buffering support
		PFD_TYPE_RGBA,					// RGBA color mode
		32,								// 32 bit color mode
		0, 0, 0, 0, 0, 0,				// ignore color bits, non-palettized mode
		0,								// no alpha buffer
		0,								// ignore shift bit
		0,								// no accumulation buffer
		0, 0, 0, 0,						// ignore accumulation bits
		16,								// 16 bit z-buffer size
		0,								// no stencil buffer
		0,								// no auxiliary buffer
		PFD_MAIN_PLANE,					// main drawing plane
		0,								// reserved
		0, 0, 0 };						// layer masks ignored

	nPixelFormat = ChoosePixelFormat(hDC, &pfd);	// choose best matching pixel format

	SetPixelFormat(hDC, nPixelFormat, &pfd);		// set pixel format to device context
}

bool InitRegisterWindowClassEx(HINSTANCE hInstance)
{
	WNDCLASSEX windowClass;		// window class

	// fill out the window class structure
	windowClass.cbSize			= sizeof(WNDCLASSEX);
	windowClass.style			= CS_HREDRAW | CS_VREDRAW;
	windowClass.lpfnWndProc		= WndProc;
	windowClass.cbClsExtra		= 0;
	windowClass.cbWndExtra		= 0;
	windowClass.hInstance		= hInstance;
	windowClass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);	// default icon
	windowClass.hCursor			= LoadCursor(NULL, IDC_ARROW);		// default arrow
	//ShowCursor(FALSE);
	windowClass.hbrBackground	= NULL;								// don't need background
	windowClass.lpszMenuName	= NULL;								// no menu
	windowClass.lpszClassName	= "MyClass";
	windowClass.hIconSm			= LoadIcon(NULL, IDI_WINLOGO);		// windows logo small icon

	// register the windows class
	if (!RegisterClassEx(&windowClass))
		return 0;

	return 1;
}

bool InitCreateWindowEx(HWND &hwnd, HINSTANCE hInstance, bool fullScreen)
{

	// Variables
	int width = 800;
	int height = 600;
	int bits = 32;
	DWORD	dwExStyle;		// Window Extended Style
	DWORD	dwStyle;		// Window Style
	RECT	windowRect;

	if (fullScreen)
	{
		windowRect.left				= (long)0;			// Set Left Value To 0
		windowRect.right			= (long)width;		// Set Right Value To Requested Width
		windowRect.top				= (long)0;			// Set Top Value To 0
		windowRect.bottom			= (long)height;		// Set Bottom Value To Requested Height
	}
	else
	{	// Needed to compensate for title bar and stupid window border crap.
		windowRect.left				= (long)0;				// Set Left Value To 0
		windowRect.right			= (long)width + 6;		// Set Right Value To Requested Width
		windowRect.top				= (long)0;				// Set Top Value To 0
		windowRect.bottom			= (long)height + 25;	// Set Bottom Value To Requested Height
	}

	if (fullScreen) {
		DEVMODE dmScreenSettings;									// device mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));
		dmScreenSettings.dmSize = sizeof(dmScreenSettings);	
		dmScreenSettings.dmPelsWidth = width;						// screen width
		dmScreenSettings.dmPelsHeight = height;						// screen height
		dmScreenSettings.dmBitsPerPel = bits;						// bits per pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// 
		if (ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL) {
			// setting display mode failed, switch to windowed
			MessageBox(NULL, "Display mode failed", NULL, MB_OK);
			fullScreen=FALSE;	
		}
	}

	if (fullScreen) {					// Are We Still In Fullscreen Mode?
		dwExStyle=WS_EX_APPWINDOW;		// Window Extended Style
		dwStyle=WS_POPUP;				// Windows Style
		//ShowCursor(FALSE);				// Hide Mouse Pointer
	} else {
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;		// Window Extended Style
		//dwStyle=WS_OVERLAPPEDWINDOW;						// Windows Style
		dwStyle=WS_OVERLAPPED;								// Windows Style
	}

	AdjustWindowRectEx(&windowRect, dwStyle, FALSE, dwExStyle);		// Adjust Window To True Requested Size

	// class registered, so now create our window
	hwnd = CreateWindowEx(NULL,											// extended style
						  "MyClass",									// class name
						  "Harbor Wars",								// app name
						  dwStyle | WS_SYSMENU | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
						  0, 0,											// x,y coordinate
						  windowRect.right - windowRect.left,
						  windowRect.bottom - windowRect.top,			// width, height
						  NULL,											// handle to parent
						  NULL,											// handle to menu
						  hInstance,									// application instance
						  NULL);										// no extra params

	// check if window creation failed (hwnd would equal NULL)
	if (!hwnd)
		return 0;

	return 1;
}


bool InitConsoleWindow()
{

	AllocConsole( );  // Create Console Window
	freopen("CONIN$","rb",stdin);   // reopen stdin handle as console window input
	freopen("CONOUT$","wb",stdout);  // reopen stout handle as console window output
	freopen("CONOUT$","wb",stderr); // reopen stderr handle as console window output

	return 1;
}
