#include "md2.h"

// vector subtraction
vector_t operator-(vector_t a, vector_t b)
{
	vector_t c;

	c.point[0] = a.point[0] - b.point[0];
	c.point[1] = a.point[1] - b.point[1];
	c.point[2] = a.point[2] - b.point[2];

	return c;
}

// scalar-vector multiplication
vector_t operator*(float f, vector_t b)
{
	vector_t c;

	c.point[0] = f * b.point[0];
	c.point[1] = f * b.point[1];
	c.point[2] = f * b.point[2];

	return c;
}

// vector division
vector_t operator/(vector_t a, vector_t b)
{
	vector_t c;

	c.point[0] = a.point[0] / b.point[0];
	c.point[1] = a.point[1] / b.point[1];
	c.point[2] = a.point[2] / b.point[2];

	return c;
}

// vector addition
vector_t operator+(vector_t a, vector_t b)
{
	vector_t c;

	c.point[0] = a.point[0] + b.point[0];
	c.point[1] = a.point[1] + b.point[1];
	c.point[2] = a.point[2] + b.point[2];

	return c;
}

// CalculateNormal()
// desc: given 3 points, calculates the normal to the points
void CalculateNormal( float *p1, float *p2, float *p3 )
{
   float a[3], b[3], result[3];
   float length;

   a[0] = p1[0] - p2[0];
   a[1] = p1[1] - p2[1];
   a[2] = p1[2] - p2[2];

   b[0] = p1[0] - p3[0];
   b[1] = p1[1] - p3[1];
   b[2] = p1[2] - p3[2];

   result[0] = a[1] * b[2] - b[1] * a[2];
   result[1] = b[0] * a[2] - a[0] * b[2];
   result[2] = a[0] * b[1] - b[0] * a[1];

   // calculate the length of the normal
   length = (float)sqrt(result[0]*result[0] + result[1]*result[1] + result[2]*result[2]);

   // normalize and specify the normal
   glNormal3f(result[0]/length, result[1]/length, result[2]/length);
}

// CMD2Model constructor
CMD2Model::CMD2Model()
{
     numVertices = 0;    // vertices
     numTriangles = 0;   // triangles
     numFrames = 0;      // frames
     numST = 0;          // texture coordinates
     frameSize = 0;      // needed?
     currentFrame = 0;   // current keyframe 
     nextFrame = 1;      // next keyframe
     interpol = 0.0;     // interpolation percent
     triIndex = NULL;    // triangle indices
     st = NULL;          // texture coordinate indices
     vertexList = NULL;  // vertices
     modelTex = NULL;    // skin/texture
	modelState = MODEL_IDLE;
}    

// CMD2Model destructor
CMD2Model::~CMD2Model()
{
}

// CMD2Model::SetupSkin()
// access: private
// desc: sets up the model skin/texture for OpenGL
void CMD2Model::SetupSkin(Texture *thisTexture)
{
	unsigned int tempID;

    // set the proper parameters for an MD2 texture
    glGenTextures(1, &tempID);
	thisTexture->setTexID(tempID);
	thisTexture->bind();
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     
    thisTexture->buildMipMaps();
}

// CMD2Model::Load()
// access: public
// desc: loads model and skin
int CMD2Model::Load(char *modelFile, char *skinFile)
{
     //FILE *filePtr;                          // file pointer
	 ifstream filePtr;
     int fileLen;                            // length of model file
     char *buffer;                           // file buffer
	 float X[2],Y[2],Z[2];
     
     modelHeader_t *modelHeader;             // model header
     
     stIndex_t *stPtr;                       // texture data
     frame_t *frame;                              // frame data
     vector_t *vertexListPtr;                // index variable
     mesh_t *bufIndexPtr;					// index variables
     int i, j;                               // index variables


     // Default min(0) and max(1) to keep track...
	 X[0] = Y[0] = Z[0] = 100.0f;
	 X[1] = Y[1] = Z[1] = -100.0f;

     // open the model file
     //filePtr = fopen(modelFile, "rb");
	 filePtr.open(modelFile, ios::binary);
     //if (filePtr == NULL)
	 if (!filePtr.is_open())
	 {
		 //printf("Could not open %s\n", modelFile);
		 cout << "Coule not open " << modelFile << endl;
          return FALSE;
	 }
	 else
		 //printf("Loaded %s\n", modelFile);
		 cout << "Loaded " << modelFile << endl;

     
     // find length of file
     //fseek(filePtr, 0, SEEK_END);
	 filePtr.seekg(0, ios::end);
     //fileLen = ftell(filePtr);
	 fileLen = filePtr.tellg();
	 //fseek(filePtr, 0, SEEK_SET);
	 filePtr.seekg(0, ios::beg);
     
     // read entire file into buffer
     buffer = new char [fileLen+1];
     //fread(buffer, sizeof(char), fileLen, filePtr);
	 filePtr.read(buffer, fileLen);
     
     // extract model file header from buffer
     modelHeader = (modelHeader_t*)buffer;
     
	 //printf("modelHeader->numXYZ = %d\n", modelHeader->numXYZ);
	 cout << "modelHeader->numXYZ = " << modelHeader->numXYZ << endl;
	 //printf("modelHeader->numFrames = %d\n", modelHeader->numFrames);
	 cout << "modelHeader->numFrames = " << modelHeader->numFrames << endl;
     vertexList = new vector_t [modelHeader->numXYZ * modelHeader->numFrames];

	 //printf("blah\n");
	 cout << "blah" << endl;
     numVertices = modelHeader->numXYZ;
	 //printf("numVertices = %d\n", numVertices);
	 cout << "numVertices = " << numVertices << endl;
     numFrames = modelHeader->numFrames;
	 //printf("numFrames = %d\n", numFrames);
	 cout << "numFrames = " << numFrames << endl;
     frameSize = modelHeader->framesize;
	 //printf("frameSize = %d\n", frameSize);
	 cout << "frameSize = " << frameSize << endl;
     

     for (j = 0; j < numFrames; j++)
     {
		frame = (frame_t*)&buffer[modelHeader->offsetFrames + frameSize * j];
		cout << "Scale = (" << frame->scale[0] << "," << frame->scale[1] << "," << frame->scale[2] << ")" << endl;
          
          vertexListPtr = (vector_t*)&vertexList[numVertices * j];
          for (i = 0; i < numVertices; i++)
          {
			  // for some reason, Milkshape swapped Y and Z...bastards.
			  // and they pointed Z in the wrong direction...damn them all to hell.
			  //
              //vertexListPtr[i].point[0] = frame->scale[0] * frame->fp[i].v[0] + frame->translate[0];
              //vertexListPtr[i].point[1] = frame->scale[1] * frame->fp[i].v[1] + frame->translate[1];
              //vertexListPtr[i].point[2] = frame->scale[2] * frame->fp[i].v[2] + frame->translate[2];
              vertexListPtr[i].point[0] = frame->scale[0] * frame->fp[i].v[0] + frame->translate[0];
              vertexListPtr[i].point[2] = frame->scale[1] * (255 - frame->fp[i].v[1]) + frame->translate[1];
              vertexListPtr[i].point[1] = frame->scale[2] * frame->fp[i].v[2] + frame->translate[2];

				// Finding minimums
				if (vertexListPtr[i].point[0] < X[0]) X[0] = vertexListPtr[i].point[0];
				if (vertexListPtr[i].point[1] < Y[0]) Y[0] = vertexListPtr[i].point[1];
				if (vertexListPtr[i].point[2] < Z[0]) Z[0] = vertexListPtr[i].point[2];
				// Finding maximums
				if (vertexListPtr[i].point[0] > X[1]) X[1] = vertexListPtr[i].point[0];
				if (vertexListPtr[i].point[1] > Y[1]) Y[1] = vertexListPtr[i].point[1];
				if (vertexListPtr[i].point[2] > Z[1]) Z[1] = vertexListPtr[i].point[2];
          }
     }

	 cout << "Maximum values: (" << X[1] << "," << Y[1] << "," << Z[1] << ")" << endl;
	 cout << "Minimum values: (" << X[0] << "," << Y[0] << "," << Z[0] << ")" << endl;

	 modelTex = new Texture;

     if (modelTex->LoadTextureFile(skinFile) != NULL)
          SetupSkin(modelTex);
     else
          return FALSE;
     
     numST = modelHeader->numST;
     st = new texCoord_t [numST];
     
     stPtr = (stIndex_t*)&buffer[modelHeader->offsetST];
     for (i = 0; i < numST; i++)
     {
          st[i].s = (float)stPtr[i].s / (float)modelTex->getWidth();
          st[i].t = (float)stPtr[i].t / (float)modelTex->getHeight();
     }
     
     numTriangles = modelHeader->numTris;
     triIndex = new mesh_t [numTriangles];
     
     // point to triangle indexes in buffer
     bufIndexPtr = (mesh_t*)&buffer[modelHeader->offsetTris];
     
     // create a mesh (triangle) list
     for (j = 0; j < numFrames; j++)         
     {
          // for all triangles in each frame
          for(i = 0; i < numTriangles; i++)
          {
               triIndex[i].meshIndex[0] = bufIndexPtr[i].meshIndex[0];
               triIndex[i].meshIndex[1] = bufIndexPtr[i].meshIndex[1];
               triIndex[i].meshIndex[2] = bufIndexPtr[i].meshIndex[2];
               triIndex[i].stIndex[0] = bufIndexPtr[i].stIndex[0];
               triIndex[i].stIndex[1] = bufIndexPtr[i].stIndex[1];
               triIndex[i].stIndex[2] = bufIndexPtr[i].stIndex[2];
          }
     }
     
     // close file and free memory
     //fclose(filePtr);
	 filePtr.close();
     free(buffer);
     
     currentFrame = 0;
     nextFrame = 1;
     interpol = 0.0;
     
     return TRUE;
}

// CMD2Model::LoadModel()
// access: public
// desc: loads model from file
int CMD2Model::LoadModel(char *modelFile)
{
     //FILE *filePtr;                          // file pointer
	 ifstream filePtr;
     int fileLen;                            // length of model file
     char *buffer;                           // file buffer
     
     modelHeader_t *modelHeader;             // model header
     
     stIndex_t *stPtr;                       // texture data
     frame_t *frame;                              // frame data
     vector_t *vertexListPtr;                // index variable
     mesh_t *triIndex, *bufIndexPtr;         // index variables
     int i, j;                               // index variables
     
     // open the model file
     //filePtr = fopen(modelFile, "rb");
	 filePtr.open(modelFile, ios::binary);
     //if (filePtr == NULL)
	 if (!filePtr.is_open())
          return FALSE;
     
     // find length of file
     //fseek(filePtr, 0, SEEK_END);
	 filePtr.seekg(0, ios::end);
     //fileLen = ftell(filePtr);
	 fileLen = filePtr.tellg();
     //fseek(filePtr, 0, SEEK_SET);
	 filePtr.seekg(0, ios::beg);
     
     // read entire file into buffer
     buffer = new char [fileLen+1];
     //fread(buffer, sizeof(char), fileLen, filePtr);
	 filePtr.read(buffer, fileLen);
     
     // extract model file header from buffer
     modelHeader = (modelHeader_t*)buffer;
     
     // allocate vertex list
     vertexList = new vector_t [modelHeader->numXYZ * modelHeader->numFrames];
     
     numVertices = modelHeader->numXYZ;
     numFrames = modelHeader->numFrames;
     frameSize = modelHeader->framesize;
     
     for (j = 0; j < numFrames; j++)
     {
          frame = (frame_t*)&buffer[modelHeader->offsetFrames + frameSize * j];
          
          vertexListPtr = (vector_t*)&vertexList[numVertices * j];
          for (i = 0; i < numVertices; i++)
          {
               vertexListPtr[i].point[0] = frame->scale[0] * frame->fp[i].v[0] + frame->translate[0];
               vertexListPtr[i].point[1] = frame->scale[1] * frame->fp[i].v[1] + frame->translate[1];
               vertexListPtr[i].point[2] = frame->scale[2] * frame->fp[i].v[2] + frame->translate[2];
          }
     }
     
     numST = modelHeader->numST;
     
     st = new texCoord_t [numST];
     
     stPtr = (stIndex_t*)&buffer[modelHeader->offsetST];
     for (i = 0; i < numST; i++)
     {
          st[i].s = 0.0;
          st[i].t = 0.0;
     }
     
     numTriangles = modelHeader->numTris;
     triIndex = new mesh_t [numTriangles];
     
     // point to triangle indexes in buffer
     bufIndexPtr = (mesh_t*)&buffer[modelHeader->offsetTris];
     
     // create a mesh (triangle) list
     for (j = 0; j < numFrames; j++)         
     {
          // for all triangles in each frame
          for(i = 0; i < numTriangles; i++)
          {
               triIndex[i].meshIndex[0] = bufIndexPtr[i].meshIndex[0];
               triIndex[i].meshIndex[1] = bufIndexPtr[i].meshIndex[1];
               triIndex[i].meshIndex[2] = bufIndexPtr[i].meshIndex[2];
               triIndex[i].stIndex[0] = bufIndexPtr[i].stIndex[0];
               triIndex[i].stIndex[1] = bufIndexPtr[i].stIndex[1];
               triIndex[i].stIndex[2] = bufIndexPtr[i].stIndex[2];
          }
     }
     
     // close file and free memory
     //fclose(filePtr);
	 filePtr.close();
	 free(buffer);
     
     modelTex = NULL;
     currentFrame = 0;
     nextFrame = 1;
     interpol = 0.0;
     
     return 0;
}

// CMD2Model::LoadSkin()
// access: public
// desc: loads a skin for the model
int CMD2Model::LoadSkin(char *skinFile)
{
     int i;
     
     if (modelTex->LoadTextureFile(skinFile) != NULL)
          SetupSkin(modelTex);
     else
          return -1;
     
     for (i = 0; i < numST; i++)
     {
          st[i].s /= (float)modelTex->getWidth();
          st[i].t /= (float)modelTex->getHeight();
     }
     
     return 0;
}

// CMD2Model::SetTexture()
// access: public
// desc: sets a new texture object 
int CMD2Model::SetTexture(Texture *texture)
{
     int i;
     
     if (texture != NULL)
	{
		free(modelTex);
          modelTex = texture;
	}
     else
          return -1;
     
     SetupSkin(modelTex);
     
     for (i = 0; i < numST; i++)
     {
          st[i].s /= (float)modelTex->getWidth();
          st[i].t /= (float)modelTex->getHeight();
     }
     
     return 0;
}

// CMD2Model::Animate()
// access: public
// desc: animates the model between the keyframes startFrame and endFrame
int CMD2Model::Animate(int startFrame, int endFrame, float percent)
{
     vector_t *vList;              // current frame vertices
     vector_t *nextVList;          // next frame vertices
     int i;                                  // index counter
     float x1, y1, z1;                  // current frame point values
     float x2, y2, z2;                  // next frame point values
     
     vector_t vertex[3]; 
     
     if ((startFrame > currentFrame))
          currentFrame = startFrame;
     
     if ((startFrame < 0) || (endFrame < 0))
          return -1;
     
     if ((startFrame >= numFrames) || (endFrame >= numFrames))
          return -1;
     
     if (interpol >= 1.0)
     {
          interpol = 0.0f;
          currentFrame++;
          if (currentFrame >= endFrame)
               currentFrame = startFrame;
          
          nextFrame = currentFrame + 1;
          
          if (nextFrame >= endFrame)
               nextFrame = startFrame;
          
     }
     
     vList = &vertexList[numVertices*currentFrame];
     nextVList = &vertexList[numVertices*nextFrame];
     
     modelTex->bind();
     glBegin(GL_TRIANGLES);
     for (i = 0; i < numTriangles; i++)
     {
          // get first points of each frame
          x1 = vList[triIndex[i].meshIndex[0]].point[0];
          y1 = vList[triIndex[i].meshIndex[0]].point[1];
          z1 = vList[triIndex[i].meshIndex[0]].point[2];
          x2 = nextVList[triIndex[i].meshIndex[0]].point[0];
          y2 = nextVList[triIndex[i].meshIndex[0]].point[1];
          z2 = nextVList[triIndex[i].meshIndex[0]].point[2];
          
          // store first interpolated vertex of triangle
          vertex[0].point[0] = x1 + interpol * (x2 - x1);
          vertex[0].point[1] = y1 + interpol * (y2 - y1);
          vertex[0].point[2] = z1 + interpol * (z2 - z1);
          
          // get second points of each frame
          x1 = vList[triIndex[i].meshIndex[2]].point[0];
          y1 = vList[triIndex[i].meshIndex[2]].point[1];
          z1 = vList[triIndex[i].meshIndex[2]].point[2];
          x2 = nextVList[triIndex[i].meshIndex[2]].point[0];
          y2 = nextVList[triIndex[i].meshIndex[2]].point[1];
          z2 = nextVList[triIndex[i].meshIndex[2]].point[2];
          
          // store second interpolated vertex of triangle
          vertex[2].point[0] = x1 + interpol * (x2 - x1);
          vertex[2].point[1] = y1 + interpol * (y2 - y1);
          vertex[2].point[2] = z1 + interpol * (z2 - z1);   
          
          // get third points of each frame
          x1 = vList[triIndex[i].meshIndex[1]].point[0];
          y1 = vList[triIndex[i].meshIndex[1]].point[1];
          z1 = vList[triIndex[i].meshIndex[1]].point[2];
          x2 = nextVList[triIndex[i].meshIndex[1]].point[0];
          y2 = nextVList[triIndex[i].meshIndex[1]].point[1];
          z2 = nextVList[triIndex[i].meshIndex[1]].point[2];
          
          // store third interpolated vertex of triangle
          vertex[1].point[0] = x1 + interpol * (x2 - x1);
          vertex[1].point[1] = y1 + interpol * (y2 - y1);
          vertex[1].point[2] = z1 + interpol * (z2 - z1);
          
          // calculate the normal of the triangle
          CalculateNormal(vertex[0].point, vertex[2].point, vertex[1].point);
          
          // render properly textured triangle
          glTexCoord2f(st[triIndex[i].stIndex[0]].s,
               st[triIndex[i].stIndex[0]].t);
          glVertex3fv(vertex[0].point);
          
          glTexCoord2f(st[triIndex[i].stIndex[2]].s ,
               st[triIndex[i].stIndex[2]].t);
          glVertex3fv(vertex[2].point);
          
          glTexCoord2f(st[triIndex[i].stIndex[1]].s,
               st[triIndex[i].stIndex[1]].t);
          glVertex3fv(vertex[1].point);
     }
     glEnd();
     
     interpol += percent;  // increase percentage of interpolation between frames
     
     return 0;
}

// RenderFrame()
// desc: renders a single key frame
int CMD2Model::RenderFrame(int keyFrame)
{
     vector_t *vList;
     int i;
    

     // create a pointer to the frame we want to show
     vList = &vertexList[numVertices * keyFrame];
     
	if (modelTex != NULL)
		// set the texture
		modelTex->bind();
     
     // display the textured model with proper lighting normals
     glBegin(GL_TRIANGLES);
     for(i = 0; i < numTriangles; i++)
     {

		  //cout << "points(" << vList[triIndex[i].meshIndex[0]].point[0] << "," << vList[triIndex[i].meshIndex[0]].point[1] << "," << vList[triIndex[i].meshIndex[0]].point[2] << ")" << endl;
		  //cout << "points(" << vList[triIndex[i].meshIndex[1]].point[0] << "," << vList[triIndex[i].meshIndex[1]].point[1] << "," << vList[triIndex[i].meshIndex[1]].point[2] << ")" << endl;
		  //cout << "points(" << vList[triIndex[i].meshIndex[2]].point[0] << "," << vList[triIndex[i].meshIndex[2]].point[1] << "," << vList[triIndex[i].meshIndex[2]].point[2] << ")" << endl;

		  CalculateNormal(vList[triIndex[i].meshIndex[0]].point,
               vList[triIndex[i].meshIndex[2]].point,
               vList[triIndex[i].meshIndex[1]].point);

		if (modelTex != NULL)
			glTexCoord2f(st[triIndex[i].stIndex[0]].s,
				st[triIndex[i].stIndex[0]].t);

          glVertex3fv(vList[triIndex[i].meshIndex[0]].point);
          
		if (modelTex != NULL)
			glTexCoord2f(st[triIndex[i].stIndex[2]].s ,
				st[triIndex[i].stIndex[2]].t);

          glVertex3fv(vList[triIndex[i].meshIndex[2]].point);

          
		if (modelTex != NULL)
			glTexCoord2f(st[triIndex[i].stIndex[1]].s,
				st[triIndex[i].stIndex[1]].t);

          glVertex3fv(vList[triIndex[i].meshIndex[1]].point);
     }
     glEnd();
     
     return 0;
}

// Unload()
// desc: unloads model data from memory
int CMD2Model::Unload()
{
     if (triIndex != NULL)
          free(triIndex);
     if (vertexList != NULL)
          free(vertexList);
     if (st != NULL)
          free(st);
     
     return 0;
}

// SetState()
// desc: set the model state
int CMD2Model::SetState(modelState_t state)
{
	modelState = state;
	return 0;
}

// GetState()
// desc: retrieve the model state
modelState_t CMD2Model::GetState()
{
	return modelState;
}
