#include <iostream>
#define _USE_MATH_DEFINES
#include <windows.h>
#include <cmath>
//#include <gl/gl.h>
#include <gl/glu.h>
//#include <gl/glaux.h>
#include "HiResTimer.h"
#include "texture.h"
#include "3ds.h"
#include "model.h"
#include "missile.h"

Missile::Missile()
{
	Init();
}

void Missile::Init()
{
	apex = 0.0f;
	alive = true;
	born = 0.0f;
	sleep_secs = 0;
	lived = 0.0f;
	m = 0.0f;
	x1 = 0.0f;
	y1 = 0.0f;
	z1 = 0.0f;
	x2 = 0.0f;
	y2 = 0.0f;
	z2 = 0.0f;
	time_counter = 0;	// not used, except in legacy Animate() function
	m_timer.Init();
}

void Missile::InitTime(float b, float secs)
{
	born = b;
	sleep_secs = secs;
}

Missile::~Missile()
{
	// do nothing for now...
}

void Missile::SetStartPoint(float x, float y, float z)
{
	x1 = x;
	y1 = y;
	z1 = z;
	//cout << "Shooting from (" << x1 << ", " << y1 << ", " << z1 << ")" << endl;
}

void Missile::SetEndPoint(float x, float y, float z)
{
	x2 = x;
	y2 = y;
	z2 = z;
	//cout << "Shooting at (" << x2 << ", " << y2 << ", " << z2 << ")" << endl;
}

void Missile::SetTargetCoord(int x, int z)
{
	target_x = x;
	target_z = z;
}

int Missile::GetTargetXCoord()
{
	return target_x;
}

int Missile::GetTargetZCoord()
{
	return target_z;
}

void Missile::SetApex(float a)
{
	apex = a;
}

void Missile::Precalculate()
{
	float mx, mz;
	float cy;
	float opp, hyp, adj;

	// distance the missile needs to travel in x and z directions
	dx = x2 - x1;
	dz = z2 - z1;

	/* Calculate middle x and middle z */
	mx = x1 + dx / 2.0f;
	mz = z1 + dz / 2.0f;

	/* Calculate Y value for apex */
	cy = - ( (mx - x1) * (mx - x2) + (mz - z1) * (mz - z2) );

	/* m is the scale value to keep apex at apex */
	m = apex / cy;

	// need dx and dz
	opp = dx;
	adj = dz;
	hyp = (float)sqrt( adj*adj + opp*opp);
	y_angle = (float)((180.0 / (float)M_PI) * (float)asin(opp / hyp));
}

// This isn't quite complete...and probably useless in this format -- see: RenderFrame()
// 
// **** THIS IS NOT USED, BUT DO NOT DELETE ****
//
void Missile::Animate()
{
	float dx = 0.0;
	float dz = 0.0;
	float slope;
	float cy;
	float mx, mz;
	float xt, yt, zt;
	float opp, hyp, adj;
	float x_angle, y_angle;

	/* Calculating step value */
	dx = (x2 - x1) / (float)( MISSILE_TIME * fps );
	dz = (z2 - z1) / (float)( MISSILE_TIME * fps );

	/* Calculate middle x and middle z */
	mx = x1 + (x2 - x1) / 2.0f;
	mz = z1 + (z2 - z1) / 2.0f;

	/* Calculate Y value for apex */
	cy = - ( (mx - x1) * (mx - x2) + (mz - z1) * (mz - z2) );

	/* m is the scale value to keep apex at apex */
	m = apex / cy;

	opp = (x2 - x1);
	adj = (z2 - z1);
	hyp = (float)sqrt( adj*adj + opp*opp);
	y_angle = (float)((180.0 / (float)M_PI) * (float)asin(opp / hyp));

	for (time_counter=0; time_counter <= (MISSILE_TIME * fps); time_counter ++)
	{
		xt = x1 + time_counter * dx;
		zt = z1 + time_counter * dz;
		yt = - ( (xt - x1) * (xt - x2) + (zt - z1) * (zt - z2) ) * m;
		slope = - ( 2 * xt - x1 - x2 + 2 * zt - z1 - z2) * m;

		opp = slope;
		hyp = (float)sqrt( 1.0 + opp*opp);
		x_angle = (float)((180.0 / (float)M_PI) * (float)asin(opp / hyp));

		glPushMatrix();
			glRotatef(x_angle, 1.0f, 0.0f, 0.0f);
			glRotatef(y_angle, 0.0f, 1.0f, 0.0f);
			//model.Render();
		glPopMatrix();
	}
}

// The missile will maintain it's own counter, and know how long it has
// to live (not unlike a particle in a particle system), and so it will
// basically place the missile in the frame buffer appropriately, and
// then exit.
short Missile::TranslateAndRotate(float now, float degrees)
{
		float t;
		float percent;
		float xt, yt, zt;
		float slope;
		float x_angle;
		float opp, hyp;

		t = now - born;

		// Don't draw, unless we're done sleeping
		if (t < sleep_secs)
			return MISSILE_SLEEPING;
		
		lived = (t - sleep_secs);

		if (alive)
		{
			// Get number of seconds elapsed since last call...
			// t = m_timer.GetElapsedSeconds();

			// Calculate the percent of the missile's entire life-span.
			percent = lived/MISSILE_TIME;

			// Calculate what x and z coordinate we'll be at
			xt = x1 + percent * dx;
			zt = z1 + percent * dz;

			// Calculate the height of the missle at this point
			yt = - ( (xt - x1) * (xt - x2) + (zt - z1) * (zt - z2) ) * m;

			// Use first derivative to calculate the slope
			slope = - ( 2 * xt - x1 - x2 + 2 * zt - z1 - z2) * m;

			// Need to use slope to calculate the pitch of the angle
			opp = slope;
			hyp = (float)sqrt( 1.0 + opp*opp);
			x_angle = (float)((180.0 / (float)M_PI) * (float)asin(opp / hyp));

			//cout << "Translating to (" << xt << ", " << yt << ", " << zt << ")" << endl;
			glTranslatef(xt, yt - 19.5f + y2, zt);
			glRotatef(x_angle + degrees, -1.0f, 0.0f, 0.0f);
			glRotatef(y_angle, 0.0f, 1.0f, 0.0f);

			if (percent > 1.0f)
				alive = false;
		}
	return (alive) ? MISSILE_FLYING : MISSILE_DEAD;
}
