#ifndef __POINTS_H
#define __POINTS_H

#ifndef PI 
#define PI        3.141592654 
#endif 


// Definition of point containing float values
//
typedef struct {
	float x;	/* X-coord of my_point */
	float y;	/* Y-coord of my_point */
	float z;	/* Z-coord of my_point */
	float w;
} Pointf;

// Definition of point containing double values
//
typedef struct {
	double x;	/* X-coord of my_point */
	double y;	/* Y-coord of my_point */
	double z;	/* Z-coord of my_point */
	double w;
} Pointd;

// Scales a vector by a factor of 'f'
//
// float version
extern void scaleVectorf(float, Pointf &);
// double version
extern void scaleVectord(double, Pointd &);

// Conversion of vectors to Unit-Vectors (Float and Double)
//
// float version
extern bool normalizeVectorf(Pointf &);
// double version
extern bool normalizeVectord(Pointd &);

// Cross Product calculation when you have only 3 points.
//
// float version
extern Pointf crossProduct3f(Pointf, Pointf, Pointf);
// double version
extern Pointd crossProduct3d(Pointd, Pointd, Pointd);

// Cross Product calculation when you have 2 vectors.
//
// float version
extern Pointf crossProduct2f(Pointf, Pointf);
// double version
extern Pointd crossProduct2d(Pointd, Pointd);

// Dot Product calculation of two vectors
//
// float version
extern float dotProduct2f(Pointf v1, Pointf v2);
// double version
extern double dotProduct2d(Pointd v1, Pointd v2);

// Calculates the angle between two vectors
//
// float version
extern float angleBetweenVectorsf(Pointf, Pointf);
// double version
extern double angleBetweenVectorsd(Pointd, Pointd);

#endif
