/*
   somepoint.cpp - v0.1
   
   the "some" point class
   
   Copyright (c) 2003 Patrick Lynch <plynch@sphere.org>
   Copyright (c) 2003 Brian Jared <bjared@g3.org>
   
   v 0.1
   - Inital concept and layout completed.
   
*/

#include "somepoint.h"

// constructor
somepoint::somepoint(void)
{
	// Point begins on origin;
	x = y = z = 0;
	
	// The colors set to 0
	r_color = g_color = b_color = a_color = 0;

}

// constructor with points
somepoint::somepoint(float x_cord, float y_cord, float z_cord)
{
	x = x_cord;
	y = y_cord;
	z = z_cord;
}

// Set the point coordinates
void somepoint::SetCord(float x_cord, float y_cord, float z_cord)
{
	x = x_cord;
	y = y_cord;
	z = z_cord;
}

// Set the point coordinates
void somepoint::SetRandCord(float min, float max, bool force)
{
	if (x == 0 || force) {
		x = (float)((1 + (max * rand() / (min + RAND_MAX))) * 4) / 10;
	}
	if (y == 0 || force) {
		y = (float)((1 + (max * rand() / (min + RAND_MAX))) * 4) / 10;
	}
	if (z == 0 || force) {
		z = (float)((1 + (max * rand() / (min + RAND_MAX))) * 4) / 10;
	}
}

// one point to another.
void somepoint::operator=(const somepoint &a)
{
	x = a.x; y = a.y; z = a.z;

	if (a.r_color > 0)
		r_color = a.r_color;
	if (a.g_color > 0)
		g_color = a.g_color;
	if (a.b_color > 0)
		b_color = a.b_color;
	if (a.a_color > 0)
		a_color = a.a_color;
}

somepoint somepoint::operator-(const somepoint &a)
{
	somepoint Diff;

	Diff.SetCord(x - a.x, y - a.y, z - a.z);

	return Diff;
}

somepoint somepoint::operator+(const somepoint &a)
{
	somepoint Sum;

	Sum.SetCord(x + a.x, y + a.y, z + a.z);

	return Sum;
}

// Set the color of the point
void somepoint::setcolor(float r, float g, float b, float a = 0.0f)
{
	r_color = r; g_color = g; b_color = b; a_color = a;
}

// Set the color of the point to a random color
void somepoint::SetRandColor(bool force)
{
	if (r_color == 0 || force) {
		r_color = (float)((1 + (255 * rand() / (1.0 + RAND_MAX))) * 4) / 1000;
	}
	if (g_color == 0 || force) {
		g_color = (float)((1 + (255 * rand() / (1.0 + RAND_MAX))) * 4) / 1000;
	}
	if (b_color == 0 || force) {
		b_color = (float)((1 + (255 * rand() / (1.0 + RAND_MAX))) * 4) / 1000;
	}
}

somepoint somepoint::MidPoint(const somepoint &Q)
{
	somepoint Mid((x + Q.x) / 2, (y + Q.y) / 2, (z + Q.z) / 2);

	return Mid;
}

// To get the difference in x between two points
float somepoint::dx(const somepoint &a)
{
	return (x - a.x);
}

// To get the difference in y between two points
float somepoint::dy(const somepoint &a)
{
	return (y - a.y);
}

// To get the difference in z between two points
float somepoint::dz(const somepoint &a)
{
	return (z - a.z);
}

// Set the color and then draw the point
void somepoint::draw(void)
{
	glColor3f(r_color, g_color, b_color);
	glVertex3f(x, y, z);
}

// Print out all parts of a point at any given moment in time to stdout
void somepoint::printout()
{
	//cout.precision(2);
	cout << "Point: " << this << "   ";
	cout << "Cord: [" << x << ", " << y << ", " << z << "]" << "   ";
	cout << "Color: [" << r_color << ", " << g_color << ", " << b_color << ", " << a_color << "]" << endl;
}

// Print out all parts of a point at any given moment in time to file pointer
void somepoint::printout(FILE * file)
{
	fprintf(file, "\nPoint: %s    ", this);
	fprintf(file, "Cord[%.2f, %.2f, %.2f]    ", x, y, z);
	fprintf(file, "Color[%.2f, %.2f, %.2f, %.2f]\n", r_color, g_color, b_color, a_color);
}

// destructor
somepoint::~somepoint(void){}