/*
   someshape.cpp - v0.1
   
   the "some" shape class
   
   Copyright (c) 2003 Patrick Lynch <plynch@sphere.org>
   Copyright (c) 2003 Brian Jared <bjared@g3.org>
   
   v 0.1
   - Inital concept and layout completed.
   
*/
#include "someshape.h"

someshape::someshape(void)
{
}

someshape::~someshape(void)
{
}

// This function draws a cone and returns an integer that is the
// index for glCallList().  Define the location, and radius and
// if you want the normals of the cone drawn.
int draw_cone(float x, float y, float z, float radius, float height, bool texture = false, bool draw_normals = false) {

	// points and vectors I will need.
	somepoint *tip, *base_point1, *base_point2;
	somevector *tangent_vector, *side_vector, *base_normal, *tip_normal, *side_normal;
	somevector *base_point1_normal, *base_point2_normal;

	// allocate memory for points and vectors
	base_point1 = new somepoint;
	base_point2 = new somepoint;
	base_point1_normal = new somevector;
	base_point2_normal = new somevector;
	side_normal = new somevector;
	side_vector = new somevector;
	tangent_vector = new somevector;
	
	// angles for drawing the cone
	float angle;
	float step_angle = (float)(2 * M_PI) / TRIANGLES;

	// get next list index
	int list_index = glGenLists(1);

	// compiling list 
	glNewList(list_index, GL_COMPILE);
		
		//Push new matrix
		glPushMatrix();

			// Places the middle of the object in the origin.  The base starts
			// on the origin on an x,z plane, so this lowers the cone 1/2 height.
			glTranslatef(x, y - (height / 2), z);
			
			//
			// TIP of the cone
			//
			tip = new somepoint(0.0f, (float)height, 0.0f);
			tip->SetRandColor(!FORCE);

			tip_normal = new somevector(0.0f, 1.0f, 0.0f);
			if (draw_normals) {
				// Set the IP so normal isn't drawn inside the cone.
				tip_normal->SetIP(*tip);
				tip_normal->normalize();
				glDisable(GL_LIGHTING);
				glBegin(GL_LINES);
					tip_normal->draw();
				glEnd();
				glEnable(GL_LIGHTING);
			}


			//
			// BASE of the cone
			//
			// The normal for the base on an x,z plane is -y.
			base_normal = new somevector(0.0f, -1.0f, 0.0f);
			if (draw_normals) {
				glDisable(GL_LIGHTING);
				glBegin(GL_LINES);
					base_normal->normalize();
					base_normal->draw();
				glEnd();
				glEnable(GL_LIGHTING);

			}

			glBegin(GL_POLYGON);

				// define one normal for the whole polygon
				base_normal->DefineAsNormal();

				// Drawing the circular base as a polygon.  I have to step around
				// with a negative angle to make sure to follow the CCW rule.
				for(angle = 0.0f; angle >= -(2 * M_PI); angle -= step_angle)
				{
					base_point1->SetRandColor(!FORCE);
					base_point1->SetCord(radius * (float)sin(angle), 0, radius * (float)cos(angle));

					if (texture) glTexCoord2f( (1 + (float)sin(angle)) / 2, (1 + (float)cos(angle)) / 2 );
					base_point1->draw();
				}	
			glEnd();


			//
			// Drawing the sides of the cone
			//
			for(angle = 0.0f; angle < (2 * M_PI); angle += step_angle)
			{
				glBegin(GL_TRIANGLES);
					
					// define the points around the circumferance similar to the base drawn above.
					// These are going to be triangles that make up the sides of the cone.
					base_point1->SetRandColor(!FORCE);
					base_point1->SetCord(radius * (float)sin(angle), 0, radius * (float)cos(angle));

					base_point2->SetRandColor(!FORCE);
					base_point2->SetCord(radius * (float)sin(angle + step_angle), 0, radius * (float)cos(angle + step_angle));

					// calculate normal for the base_point1
					side_vector->calculate(*tip, *base_point1);
					tangent_vector->calculate((float)-cos(angle), 0, (float)sin(angle));
					*base_point1_normal = tangent_vector->CrossProduct(*side_vector);

					// calculate normal for the base point2
					side_vector->calculate(*tip, *base_point2);
					tangent_vector->calculate((float)-cos(angle + step_angle), 0, (float)sin(angle + step_angle));
					*base_point2_normal = tangent_vector->CrossProduct(*side_vector);
					
					// Normal for the whole triangle
					side_normal->CrossProduct(*tip, *base_point1, *base_point2);
					side_normal->DefineAsNormal();

					// defining a point normal for the tip
					tip_normal->DefineAsNormal();
					if (texture) glTexCoord2f(0.5f, 1.0f);
					tip->draw();

					// draw base_point1 of triangle
					base_point1_normal->DefineAsNormal();
					if (texture) glTexCoord2f( angle / (2 * M_PI), 0.0f);
					base_point1->draw();

					// draw base_point1 of triangle
					base_point2_normal->DefineAsNormal();
					if (texture)glTexCoord2f( (angle + step_angle) / (2 * M_PI)  , 0.0f);
					base_point2->draw();

				glEnd();

				if (draw_normals) {
					// To draw normals of the cone
					glDisable(GL_LIGHTING);
					glBegin(GL_LINES);
						base_point1_normal->SetIP(*base_point1);
						base_point2_normal->SetIP(*base_point2);
						side_normal->draw();
						base_point1_normal->draw();
						base_point2_normal->draw();
					glEnd();
					glEnable(GL_LIGHTING);
				}
			
			}	

		glPopMatrix();

	glEndList();

	delete tip, base_point1, base_point2;
	delete tangent_vector, side_vector, base_normal, tip_normal, side_normal;
	delete base_point1_normal, base_point2_normal;

	return list_index;

};


// This function will draw a pyramid at a specified x,y,z.  I have hardcoded
// the points involved, but then translate to the correct coordinates.
int draw_pyramid(float x, float y, float z, bool draw_normals = false ) {

	// points used in triangle drawing.
	static somepoint a, b, c, d, e;
	somevector CrossP;

	// Hardcode x, y, z for all the points
	a.SetCord(0.0f, 0.0f, 0.0f);
	b.SetCord(1.0f, 0.0f, 0.0f);
	c.SetCord(1.0f, 0.0f, 1.0f);
	d.SetCord(0.0f, 0.0f, 1.0f);
	e.SetCord(0.5f, 1.0f, 0.5f);

	// set the colors	
	a.SetRandColor(!FORCE);
	b.SetRandColor(!FORCE);
	c.SetRandColor(!FORCE);
	d.SetRandColor(!FORCE);
	e.SetRandColor(!FORCE);
	
	// get next list index
	int list_index = glGenLists(1);

	// compiling list 
	glNewList(list_index, GL_COMPILE);

		// Push new matrix
		glPushMatrix();

			// Places the object in the world relative to the coordinates
			// passed into draw_pyramid.  Offset for hard coded points is
			// already accounted for... hence the -0.5 on all axis.
			glTranslatef(x - 0.5f, y - 0.5f, z - 0.5f);

			// Used to draw normal of the base
			if (draw_normals) {
				glDisable(GL_LIGHTING);
				glBegin(GL_LINES);
					CrossP.CrossProduct(a, b, c);
					CrossP.normalize();
					CrossP.draw();
				glEnd();
				glEnable(GL_LIGHTING);
			}

			// Drawing base of the the pyramid
			glBegin(GL_QUADS);
				CrossP.CrossProduct(a, b, c);
				CrossP.DefineAsNormal();
				a.draw(); b.draw(); c.draw(); d.draw();
			glEnd();

			// Used to draw normals of the sides
			if (draw_normals) {
				glDisable(GL_LIGHTING);
				glBegin(GL_LINES);
					CrossP.CrossProduct(d, c, e);
					CrossP.normalize();
					CrossP.draw();

					CrossP.CrossProduct(c, b, e);
					CrossP.normalize();
					CrossP.draw();

					CrossP.CrossProduct(b, a, e);
					CrossP.normalize();
					CrossP.draw();

					CrossP.CrossProduct(a, d, e);
					CrossP.normalize();
					CrossP.draw();
				glEnd();
				glEnable(GL_LIGHTING);
			}

			glBegin(GL_TRIANGLES);
				CrossP.CrossProduct(d, c, e);
				CrossP.DefineAsNormal();
				d.draw(); c.draw(); e.draw();			// front face

				CrossP.CrossProduct(c, b, e);
				CrossP.DefineAsNormal();
				c.draw(); b.draw(); e.draw();			// right face

				CrossP.CrossProduct(b, a, e);
				CrossP.DefineAsNormal();
				b.draw(); a.draw(); e.draw();			// back face

				CrossP.CrossProduct(a, d, e);
				CrossP.DefineAsNormal();
				a.draw(); d.draw(); e.draw();			// Left face
			glEnd();

		glPopMatrix();

	glEndList();

	return list_index;

}