/*
   somevector.cpp - v0.1
   
   the "some" vector class
   
   Copyright (c) 2003 Patrick Lynch <plynch@sphere.org>
   Copyright (c) 2003 Brian Jared <bjared@g3.org>
   
   v 0.1
   - Inital concept and layout completed.
   
*/
#include "somevector.h"

somevector::somevector(void)
{
	length = 0;
}

somevector::somevector(float x, float y, float z)
{
	calculate(x, y, z);
}

somevector::somevector(somepoint &P)
{
	calculate(P);
}

somevector::somevector(somepoint &P, somepoint &Q)
{
	calculate(P, Q);
}

// initialize a vector object given x, y, z.  Since we only have the
// input for one point, the IP is set at the origin.
void somevector::calculate(float x, float y, float z)
{
	//cout << "Three points to calc: [" << x << "," << y << "," << z << "]" << endl;
	
	// instanciate the point objects.
	IP = IP;
	TP.SetCord(x, y, z);

	// set the direction coordinates
	dir = TP;

	// calculate the length of the vector
	length = (float)sqrt((x * x) + (y * y) + (z * z));
}

// same as above but accepting a somepoint
void somevector::calculate(somepoint &P)
{
	float x, y, z;

	x = P.getx();
	y = P.getx();
	z = P.getx();

	calculate(x, y, z);
}

// initialize a vector object when given two points.
void somevector::calculate(somepoint &P, somepoint &Q)
{
	float dx;
	float dy;
	float dz;

	// use the somepoint overloaded = operator to set the points in the vector
	IP = P;
	TP = Q;

	// The direction vector
	dir = Q - P;

	// Get the difference of each coordinate for the dir point and length.
	dx = Xdir();
	dy = Ydir();
	dz = Zdir();

	length = (float)sqrt((dx * dx) + (dy * dy) + (dz * dz));
}



somevector somevector::operator+ (somevector &Q)
{
	somevector Diff;

	Diff.calculate( Xdir() + Q.Xdir(),
					Ydir() + Q.Ydir(),
					Zdir() + Q.Zdir() );

	return Diff;
}

somevector somevector::operator- (somevector &Q)
{
	somevector Diff;

	Diff.calculate( Xdir() - Q.Xdir(),
					Ydir() - Q.Ydir(),
					Zdir() - Q.Zdir() );

	return Diff;
}

float somevector::DotProduct(somevector &Q)
{
	return ( (Xdir() * Q.Xdir()) + (Ydir() * Q.Ydir()) + (Zdir() * Q.Zdir()) );
}

somevector somevector::CrossProduct(somevector &Q)
{
	somevector Perpendicular;

	Perpendicular.calculate( (Ydir() * Q.Zdir()) - (Zdir() * Q.Ydir()),
							 (Zdir() * Q.Xdir()) - (Xdir() * Q.Zdir()),
							 (Xdir() * Q.Ydir()) - (Ydir() * Q.Xdir()) );


	Perpendicular.IP = IP.MidPoint(TP).MidPoint(Perpendicular.TP);

	//cout << "The three vectors" << endl;
	//printout();
	//Q.printout();
	//Perpendicular.printout();

	return Perpendicular;
}

void somevector::CrossProduct(somepoint &P, somepoint &Q, somepoint &R)
{
	somevector v1, v2;

	v1.calculate(P, Q);
	v2.calculate(P, R);

	calculate( (v1.Ydir() * v2.Zdir()) - (v1.Zdir() * v2.Ydir()),
	    	   (v1.Zdir() * v2.Xdir()) - (v1.Xdir() * v2.Zdir()),
			   (v1.Xdir() * v2.Ydir()) - (v1.Ydir() * v2.Xdir()) );

	IP = P.MidPoint(Q).MidPoint(R);

}

void somevector::normalize()
{
	if (length != 0) {
		calculate(Xdir() / length, Ydir() / length, Zdir() / length);
	}
}

void somevector::DefineAsNormal()
{
	normalize();
	glNormal3f(Xdir(), Ydir(), Zdir());
}

void somevector::draw()
{
	IP.setcolor(0.6f, 0.6f, 0.6f, 0.0f);
	IP.draw();
	(IP + dir).draw();
}

void somevector::printout()
{
	cout << endl <<  "Somevector: " << this << endl;
	cout << "     Init Point: "; IP.printout();
	cout << "     Term Point: "; TP.printout();
	cout << "      Direction: "; dir.printout();
	cout << "         Length: " << length << endl;
}

somevector::~somevector(void)
{
}

