#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include "images.h"
#include "texture.h"

Texture::Texture()
{
	// Do nothing.
}

// Constructor with filename argument
Texture::Texture(char *filename)
{
	if(!LoadTextureFile(filename))
		printf("Could not load texture '%s'.\n", filename);
}

// Constructor with filename argument
Texture::Texture(char *filename, bool m)
{
	if(!LoadTextureFile(filename, m))
		printf("Could not load texture '%s'.\n", filename);
}

// Destructor
Texture::~Texture()
{
	free(data);
}

unsigned int Texture::GetTexID()
{
	return texID;
}

// LoadTextureFile
bool Texture::LoadTextureFile(char *filename)
{
	BITMAPINFOHEADER	bitmapInfoHeader;
	TGAFILE * tmp_tga;
	printf("loading %s\n", filename);

	if (strstr(filename, ".bmp") || strstr(filename, ".BMP"))
	{
		Init();
		data = LoadBitmapFile(filename, &bitmapInfoHeader);
		width = bitmapInfoHeader.biWidth;
		height = bitmapInfoHeader.biHeight;
		type = BMP;
		Bind();
		BuildMipMaps();
	}
	else
	{
		if (strstr(filename, ".tga") || strstr(filename, ".TGA"))
		{
			Init();
			tmp_tga = (TGAFILE*)malloc(sizeof(TGAFILE));
			LoadTGAFile(filename, tmp_tga);
			data = tmp_tga->imageData;
			width = tmp_tga->imageWidth;
			height = tmp_tga->imageHeight;
			type = TGA;
			Bind();
			BuildMipMaps();
		}
		else
		{
			return false;
		}
	}

	return true;
}

// LoadTextureFile
bool Texture::LoadTextureFile(char *filename, bool mipmap)
{
	BITMAPINFOHEADER	bitmapInfoHeader;
	TGAFILE * tmp_tga;
	printf("loading %s\n", filename);

	if (strstr(filename, ".bmp") || strstr(filename, ".BMP"))
	{
		Init();
		data = LoadBitmapFile(filename, &bitmapInfoHeader);
		width = bitmapInfoHeader.biWidth;
		height = bitmapInfoHeader.biHeight;
		type = BMP;
		Bind();
		if (mipmap)
			BuildMipMaps();
		else
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
	}
	else
	{
		if (strstr(filename, ".tga") || strstr(filename, ".TGA"))
		{
			Init();
			tmp_tga = (TGAFILE*)malloc(sizeof(TGAFILE));
			LoadTGAFile(filename, tmp_tga);
			data = tmp_tga->imageData;
			width = tmp_tga->imageWidth;
			height = tmp_tga->imageHeight;
			type = TGA;
			Bind();

			if (mipmap)
				BuildMipMaps();
			else
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
		}
		else
		{
			return false;
		}
	}

	return true;
}

unsigned char * Texture::GetData()
{
	return data;
}

int Texture::GetWidth()
{
	return width;
}

int Texture::GetHeight()
{
	return height;
}

void Texture::SetTexID(unsigned int tempID)
{
	texID = tempID;
}

imgTypes_t Texture::GetType()
{
	return type;
}

void Texture::Init()
{
	glGenTextures(1, &texID);
}

void Texture::BuildMipMaps()
{
	switch(type)
	{
	case BMP:
		Bind();
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, width, height, GL_RGB, GL_UNSIGNED_BYTE, data);
		break;
	case PCX:
		Bind();
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);
	case TGA:
		Bind();
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);
		break;
	default:
		break;
	}
}

void Texture::TexGen(GLint mode)
{
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, mode);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, mode);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
}

void Texture::Bind()
{
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
}

